@@ -10,8 +10,8 @@ from scipy.sparse.linalg import inv
 
 def has_bc_of_form(symbol, side, bcs, form):
 
-    if symbol.id in bcs:
-        if bcs[symbol.id][side][1] == form:
+    if symbol in bcs:
+        if bcs[symbol][side][1] == form:
             return True
         else:
             return False
@@ -94,7 +94,13 @@ class Discretisation(object):
         # reset discretised_symbols
         self._discretised_symbols = {}
 
-    def process_model(self, model, inplace=True, check_model=True):
+    def process_model(
+        self,
+        model,
+        inplace=True,
+        check_model=True,
+        remove_independent_variables_from_rhs=True,
+    ):
         """Discretise a model.
         Currently inplace, could be changed to return a new model.
 
@@ -109,9 +115,15 @@ class Discretisation(object):
         check_model : bool, optional
             If True, model checks are performed after discretisation. For large
             systems these checks can be slow, so can be skipped by setting this
-            option to False. When developing, testing or debugging it is recommened
+            option to False. When developing, testing or debugging it is recommended
             to leave this option as True as it may help to identify any errors.
             Default is True.
+        remove_independent_variables_from_rhs : bool, optional
+            If True, model checks to see whether any variables from the RHS are used
+            in any other equation. If a variable meets all of the following criteria
+            (not used anywhere in the model, len(rhs)>1), then the variable
+            is moved to be explicitly integrated when called by the solution object.
+            Default is True.
 
         Returns
         -------
@@ -148,7 +160,12 @@ class Discretisation(object):
 
         # Prepare discretisation
         # set variables (we require the full variable not just id)
+
+        # Search Equations for Independence
+        if remove_independent_variables_from_rhs:
+            model = self.remove_independent_variables_from_rhs(model)
         variables = list(model.rhs.keys()) + list(model.algebraic.keys())
+        # Find those RHS's that are constant
         if self.spatial_methods == {} and any(var.domain != [] for var in variables):
             for var in variables:
                 if var.domain != []:
@@ -182,8 +199,8 @@ class Discretisation(object):
             # since they point to the same object
             model_disc = model
         else:
-            # create an empty copy of the original model
-            model_disc = model.new_empty_copy()
+            # create a copy of the original model
+            model_disc = model.new_copy()
 
         # Keep a record of y_slices in the model
         model_disc.y_slices = self.y_slices_explicit
@@ -236,6 +253,16 @@ class Discretisation(object):
             model_disc
         )
 
+        # Process length scales
+        new_length_scales = {}
+        for domain, scale in model.length_scales.items():
+            new_scale = self.process_symbol(scale)
+            if isinstance(new_scale, pybamm.Array):
+                # Convert possible arrays of length 1 to scalars
+                new_scale = pybamm.Scalar(float(new_scale.evaluate()))
+            new_length_scales[domain] = new_scale
+        model_disc._length_scales = new_length_scales
+
         # Check that resulting model makes sense
         if check_model:
             pybamm.logger.verbose("Performing model checks for {}".format(model.name))
@@ -282,7 +309,7 @@ class Discretisation(object):
                         for domain_mesh in mesh:
                             end += domain_mesh.npts_for_broadcast_to_nodes
                         # Add to slices
-                        y_slices[child.id].append(slice(start_, end))
+                        y_slices[child].append(slice(start_, end))
                         y_slices_explicit[child].append(slice(start_, end))
                         # Increment start_
                         start_ = end
@@ -290,7 +317,7 @@ class Discretisation(object):
                 end += self._get_variable_size(variable)
 
             # Add to slices
-            y_slices[variable.id].append(slice(start, end))
+            y_slices[variable].append(slice(start, end))
             y_slices_explicit[variable].append(slice(start, end))
             # Add to bounds
             lower_bounds.extend([variable.bounds[0]] * (end - start))
@@ -317,9 +344,7 @@ class Discretisation(object):
         else:
             size = 0
             spatial_method = self.spatial_methods[variable.domain[0]]
-            repeats = spatial_method._get_auxiliary_domain_repeats(
-                variable.auxiliary_domains
-            )
+            repeats = spatial_method._get_auxiliary_domain_repeats(variable.domains)
             for dom in variable.domain:
                 size += spatial_method.mesh[dom].npts_for_broadcast_to_nodes * repeats
             return size
@@ -355,7 +380,7 @@ class Discretisation(object):
             # Find the name in the model variables
             # Look up dictionary key based on value
             try:
-                idx = [x.id for x in model.variables.values()].index(var.id)
+                idx = list(model.variables.values()).index(var)
             except ValueError:
                 raise ValueError(
                     """
@@ -415,8 +440,7 @@ class Discretisation(object):
                 left_symbol_disc, right_symbol_disc, left_mesh, right_mesh
             )
 
-        # bc_key_ids = [key.id for key in list(model.boundary_conditions.keys())]
-        bc_key_ids = list(self.bcs.keys())
+        bc_keys = list(self.bcs.keys())
 
         internal_bcs = {}
         for var in model.boundary_conditions.keys():
@@ -426,24 +450,24 @@ class Discretisation(object):
                 first_child = children[0]
                 next_child = children[1]
 
-                lbc = self.bcs[var.id]["left"]
+                lbc = self.bcs[var]["left"]
                 rbc = (boundary_gradient(first_child, next_child), "Neumann")
 
-                if first_child.id not in bc_key_ids:
-                    internal_bcs.update({first_child.id: {"left": lbc, "right": rbc}})
+                if first_child not in bc_keys:
+                    internal_bcs.update({first_child: {"left": lbc, "right": rbc}})
 
                 for current_child, next_child in zip(children[1:-1], children[2:]):
                     lbc = rbc
                     rbc = (boundary_gradient(current_child, next_child), "Neumann")
-                    if current_child.id not in bc_key_ids:
+                    if current_child not in bc_keys:
                         internal_bcs.update(
-                            {current_child.id: {"left": lbc, "right": rbc}}
+                            {current_child: {"left": lbc, "right": rbc}}
                         )
 
                 lbc = rbc
-                rbc = self.bcs[var.id]["right"]
-                if children[-1].id not in bc_key_ids:
-                    internal_bcs.update({children[-1].id: {"left": lbc, "right": rbc}})
+                rbc = self.bcs[var]["right"]
+                if children[-1] not in bc_keys:
+                    internal_bcs.update({children[-1]: {"left": lbc, "right": rbc}})
 
         self.bcs.update(internal_bcs)
 
@@ -496,7 +520,7 @@ class Discretisation(object):
         # process and set pybamm.variables first incase required
         # in discrisation of other boundary conditions
         for key, bcs in model.boundary_conditions.items():
-            processed_bcs[key.id] = {}
+            processed_bcs[key] = {}
 
             # check if the boundary condition at the origin for sphere domains is other
             # than no flux
@@ -520,7 +544,7 @@ class Discretisation(object):
                 eqn, typ = bc
                 pybamm.logger.debug("Discretise {} ({} bc)".format(key, side))
                 processed_eqn = self.process_symbol(eqn)
-                processed_bcs[key.id][side] = (processed_eqn, typ)
+                processed_bcs[key][side] = (processed_eqn, typ)
 
         return processed_bcs
 
@@ -653,7 +677,7 @@ class Discretisation(object):
         model_variables = model.rhs.keys()
         model_slices = []
         for v in model_variables:
-            model_slices.append(self.y_slices[v.id][0])
+            model_slices.append(self.y_slices[v][0])
         sorted_model_variables = [
             v for _, v in sorted(zip(model_slices, model_variables))
         ]
@@ -704,62 +728,6 @@ class Discretisation(object):
 
         return mass_matrix, mass_matrix_inv
 
-    def create_jacobian(self, model):
-        """Creates Jacobian of the discretised model.
-        Note that the model is assumed to be of the form M*y_dot = f(t,y), where
-        M is the (possibly singular) mass matrix. The Jacobian is df/dy.
-
-        Note: At present, calculation of the Jacobian is deferred until after
-        simplification, since it is much faster to compute the Jacobian of the
-        simplified model. However, in some use cases (e.g. running the same
-        model multiple times but with different parameters) it may be more
-        efficient to compute the Jacobian once, before simplification, so that
-        parameters in the Jacobian can be updated (see PR #670).
-
-        Parameters
-        ----------
-        model : :class:`pybamm.BaseModel`
-            Discretised model. Must have attributes rhs, initial_conditions and
-            boundary_conditions (all dicts of {variable: equation})
-
-        Returns
-        -------
-        :class:`pybamm.Concatenation`
-            The expression trees corresponding to the Jacobian of the model
-        """
-        # create state vector to differentiate with respect to
-        y = pybamm.StateVector(slice(0, np.size(model.concatenated_initial_conditions)))
-        # set up Jacobian object, for re-use of dict
-        jacobian = pybamm.Jacobian()
-
-        # calculate Jacobian of rhs by equation
-        jac_rhs_eqn_dict = {}
-        for eqn_key, eqn in model.rhs.items():
-            pybamm.logger.debug(
-                "Calculating block of Jacobian for {!r}".format(eqn_key.name)
-            )
-            jac_rhs_eqn_dict[eqn_key] = jacobian.jac(eqn, y)
-        jac_rhs = self._concatenate_in_order(jac_rhs_eqn_dict, sparse=True)
-
-        # calculate Jacobian of algebraic by equation
-        jac_algebraic_eqn_dict = {}
-        for eqn_key, eqn in model.algebraic.items():
-            pybamm.logger.debug(
-                "Calculating block of Jacobian for {!r}".format(eqn_key.name)
-            )
-            jac_algebraic_eqn_dict[eqn_key] = jacobian.jac(eqn, y)
-        jac_algebraic = self._concatenate_in_order(jac_algebraic_eqn_dict, sparse=True)
-
-        # full Jacobian
-        if model.rhs.keys() and model.algebraic.keys():
-            jac = pybamm.SparseStack(jac_rhs, jac_algebraic)
-        elif not model.algebraic.keys():
-            jac = jac_rhs
-        else:
-            jac = jac_algebraic
-
-        return jac, jac_rhs, jac_algebraic
-
     def process_dict(self, var_eqn_dict):
         """Discretise a dictionary of {variable: equation}, broadcasting if necessary
         (can be model.rhs, model.algebraic, model.initial_conditions or
@@ -782,12 +750,8 @@ class Discretisation(object):
         for eqn_key, eqn in var_eqn_dict.items():
             # Broadcast if the equation evaluates to a number (e.g. Scalar)
             if np.prod(eqn.shape_for_testing) == 1 and not isinstance(eqn_key, str):
-                eqn = pybamm.FullBroadcast(
-                    eqn, eqn_key.domain, eqn_key.auxiliary_domains
-                )
+                eqn = pybamm.FullBroadcast(eqn, broadcast_domains=eqn_key.domains)
 
-            # note we are sending in the key.id here so we don't have to
-            # keep calling .id
             pybamm.logger.debug("Discretise {!r}".format(eqn_key))
 
             processed_eqn = self.process_symbol(eqn)
@@ -812,10 +776,10 @@ class Discretisation(object):
 
         """
         try:
-            return self._discretised_symbols[symbol.id]
+            return self._discretised_symbols[symbol]
         except KeyError:
             discretised_symbol = self._process_symbol(symbol)
-            self._discretised_symbols[symbol.id] = discretised_symbol
+            self._discretised_symbols[symbol] = discretised_symbol
             discretised_symbol.test_shape()
 
             # Assign mesh as an attribute to the processed variable
@@ -824,9 +788,9 @@ class Discretisation(object):
             else:
                 discretised_symbol.mesh = None
             # Assign secondary mesh
-            if "secondary" in symbol.auxiliary_domains:
+            if symbol.domains["secondary"] != []:
                 discretised_symbol.secondary_mesh = self.mesh.combine_submeshes(
-                    *symbol.auxiliary_domains["secondary"]
+                    *symbol.domains["secondary"]
                 )
             else:
                 discretised_symbol.secondary_mesh = None
@@ -937,10 +901,7 @@ class Discretisation(object):
                     out = disc_child * pybamm.Vector([1])
                 else:
                     out = spatial_method.broadcast(
-                        disc_child,
-                        symbol.domain,
-                        symbol.auxiliary_domains,
-                        symbol.broadcast_type,
+                        disc_child, symbol.domains, symbol.broadcast_type
                     )
                 return out
 
@@ -975,16 +936,15 @@ class Discretisation(object):
 
         elif isinstance(symbol, pybamm.VariableDot):
             return pybamm.StateVectorDot(
-                *self.y_slices[symbol.get_variable().id],
-                domain=symbol.domain,
-                auxiliary_domains=symbol.auxiliary_domains,
+                *self.y_slices[symbol.get_variable()],
+                domains=symbol.domains,
             )
 
         elif isinstance(symbol, pybamm.Variable):
             # Check if variable is a standard variable or an external variable
-            if any(symbol.id == var.id for var in self.external_variables.values()):
+            if any(symbol == var for var in self.external_variables.values()):
                 # Look up dictionary key based on value
-                idx = [x.id for x in self.external_variables.values()].index(symbol.id)
+                idx = list(self.external_variables.values()).index(symbol)
                 name, parent_and_slice = list(self.external_variables.keys())[idx]
                 if parent_and_slice is None:
                     # Variable didn't come from a concatenation so we can just create a
@@ -992,8 +952,7 @@ class Discretisation(object):
                     return pybamm.ExternalVariable(
                         symbol.name,
                         size=self._get_variable_size(symbol),
-                        domain=symbol.domain,
-                        auxiliary_domains=symbol.auxiliary_domains,
+                        domains=symbol.domains,
                     )
                 else:
                     # We have to use a special name since the concatenation doesn't have
@@ -1002,19 +961,17 @@ class Discretisation(object):
                     ext = pybamm.ExternalVariable(
                         name,
                         size=self._get_variable_size(parent),
-                        domain=parent.domain,
-                        auxiliary_domains=parent.auxiliary_domains,
+                        domains=parent.domains,
                     )
                     out = pybamm.Index(ext, slice(start, end))
-                    out.domain = symbol.domain
+                    out.copy_domains(symbol)
                     return out
-
             else:
                 # add a try except block for a more informative error if a variable
                 # can't be found. This should usually be caught earlier by
                 # model.check_well_posedness, but won't be if debug_mode is False
                 try:
-                    y_slices = self.y_slices[symbol.id]
+                    y_slices = self.y_slices[symbol]
                 except KeyError:
                     raise pybamm.ModelError(
                         """
@@ -1025,11 +982,7 @@ class Discretisation(object):
                             symbol.name
                         )
                     )
-                return pybamm.StateVector(
-                    *y_slices,
-                    domain=symbol.domain,
-                    auxiliary_domains=symbol.auxiliary_domains,
-                )
+                return pybamm.StateVector(*y_slices, domains=symbol.domains)
 
         elif isinstance(symbol, pybamm.SpatialVariable):
             return spatial_method.spatial_variable(symbol)
@@ -1040,21 +993,16 @@ class Discretisation(object):
             return new_symbol
 
         elif isinstance(symbol, pybamm.InputParameter):
-            # Return a new copy of the input parameter, but set the expected size
-            # according to the domain of the input parameter
-            expected_size = self._get_variable_size(symbol)
-            new_input_parameter = symbol.new_copy()
-            new_input_parameter.set_expected_size(expected_size)
-            return new_input_parameter
-
+            if symbol.domain != []:
+                expected_size = self._get_variable_size(symbol)
+            else:
+                expected_size = None
+            if symbol._expected_size is None:
+                symbol._expected_size = expected_size
+            return symbol.create_copy()
         else:
-            # Backup option: return new copy of the object
-            try:
-                return symbol.new_copy()
-            except NotImplementedError:
-                raise NotImplementedError(
-                    "Cannot discretise symbol of type '{}'".format(type(symbol))
-                )
+            # Backup option: return the object
+            return symbol
 
     def concatenate(self, *symbols, sparse=False):
         if sparse:
@@ -1097,19 +1045,19 @@ class Discretisation(object):
                 unpacked_variables.extend([symbol] + [var for var in symbol.children])
             else:
                 unpacked_variables.append(symbol)
-            slices.append(self.y_slices[symbol.id][0])
+            slices.append(self.y_slices[symbol][0])
 
         if check_complete:
             # Check keys from the given var_eqn_dict against self.y_slices
-            ids = {v.id for v in unpacked_variables}
-            external_id = {v.id for v in self.external_variables.values()}
+            unpacked_variables_set = set(unpacked_variables)
+            external_vars = set(self.external_variables.values())
             for var in self.external_variables.values():
-                child_ids = {child.id for child in var.children}
-                external_id = external_id.union(child_ids)
+                child_vars = set(var.children)
+                external_vars = external_vars.union(child_vars)
             y_slices_with_external_removed = set(self.y_slices.keys()).difference(
-                external_id
+                external_vars
             )
-            if ids != y_slices_with_external_removed:
+            if unpacked_variables_set != y_slices_with_external_removed:
                 given_variable_names = [v.name for v in var_eqn_dict.keys()]
                 raise pybamm.ModelError(
                     "Initial conditions are insufficient. Only "
@@ -1209,3 +1157,57 @@ class Discretisation(object):
                             var.shape, model.rhs[rhs_var].shape, var
                         )
                     )
+
+    def is_variable_independent(self, var, all_vars_in_eqns):
+        pybamm.logger.verbose("Removing independent blocks.")
+        if not isinstance(var, pybamm.Variable):
+            return False
+
+        this_var_is_independent = not (var.name in all_vars_in_eqns)
+        not_in_y_slices = not (var in list(self.y_slices.keys()))
+        not_in_discretised = not (var in list(self._discretised_symbols.keys()))
+        is_0D = len(var.domain) == 0
+        this_var_is_independent = (
+            this_var_is_independent and not_in_y_slices and not_in_discretised and is_0D
+        )
+        return this_var_is_independent
+
+    def remove_independent_variables_from_rhs(self, model):
+        rhs_vars_to_search_over = list(model.rhs.keys())
+        unpacker = pybamm.SymbolUnpacker(pybamm.Variable)
+        eqns_to_check = (
+            list(model.rhs.values())
+            + list(model.algebraic.values())
+            + [
+                x[side][0]
+                for x in model.boundary_conditions.values()
+                for side in x.keys()
+            ]
+            # only check children of variables, this will skip the variable itself
+            # and catch any other cases
+            + [child for var in model.variables.values() for child in var.children]
+        )
+        all_vars_in_eqns = unpacker.unpack_list_of_symbols(eqns_to_check)
+        all_vars_in_eqns = [var.name for var in all_vars_in_eqns]
+
+        for var in rhs_vars_to_search_over:
+            this_var_is_independent = self.is_variable_independent(
+                var, all_vars_in_eqns
+            )
+            if this_var_is_independent:
+                if len(model.rhs) != 1:
+                    pybamm.logger.info("removing variable {} from rhs".format(var))
+                    my_initial_condition = model.initial_conditions[var]
+                    model.variables[var.name] = pybamm.ExplicitTimeIntegral(
+                        model.rhs[var], my_initial_condition
+                    )
+                    # edge case where a variable appears
+                    # in variables twice under different names
+                    for key in model.variables:
+                        if model.variables[key] == var:
+                            model.variables[key] = model.variables[var.name]
+                    del model.rhs[var]
+                    del model.initial_conditions[var]
+                else:
+                    break
+        return model
