@@ -102,24 +102,11 @@ class BinaryOperator(pybamm.Symbol):
         """
         return self._binary_evaluate(left, right)
 
-    def evaluate(self, t=None, y=None, y_dot=None, inputs=None, known_evals=None):
+    def evaluate(self, t=None, y=None, y_dot=None, inputs=None):
         """See :meth:`pybamm.Symbol.evaluate()`."""
-        if known_evals is not None:
-            id = self.id
-            try:
-                return known_evals[id], known_evals
-            except KeyError:
-                left, known_evals = self.left.evaluate(t, y, y_dot, inputs, known_evals)
-                right, known_evals = self.right.evaluate(
-                    t, y, y_dot, inputs, known_evals
-                )
-                value = self._binary_evaluate(left, right)
-                known_evals[id] = value
-                return value, known_evals
-        else:
-            left = self.left.evaluate(t, y, y_dot, inputs)
-            right = self.right.evaluate(t, y, y_dot, inputs)
-            return self._binary_evaluate(left, right)
+        left = self.left.evaluate(t, y, y_dot, inputs)
+        right = self.right.evaluate(t, y, y_dot, inputs)
+        return self._binary_evaluate(left, right)
 
     def _evaluate_for_shape(self):
         """See :meth:`pybamm.Symbol.evaluate_for_shape()`."""
@@ -181,8 +168,8 @@ class Power(BinaryOperator):
         diff = exponent * (base ** (exponent - 1)) * base.diff(variable)
         # derivative if variable is in the exponent (rare, check separately to avoid
         # unecessarily big tree)
-        if any(variable.id == x.id for x in exponent.pre_order()):
-            diff += (base ** exponent) * pybamm.log(base) * exponent.diff(variable)
+        if any(variable == x for x in exponent.pre_order()):
+            diff += (base**exponent) * pybamm.log(base) * exponent.diff(variable)
         return diff
 
     def _binary_jac(self, left_jac, right_jac):
@@ -192,7 +179,7 @@ class Power(BinaryOperator):
         if right.evaluates_to_constant_number():
             return (right * left ** (right - 1)) * left_jac
         elif left.evaluates_to_constant_number():
-            return (left ** right * pybamm.log(left)) * right_jac
+            return (left**right * pybamm.log(left)) * right_jac
         else:
             return (left ** (right - 1)) * (
                 right * left_jac + left * pybamm.log(left) * right_jac
@@ -202,7 +189,7 @@ class Power(BinaryOperator):
         """See :meth:`pybamm.BinaryOperator._binary_evaluate()`."""
         # don't raise RuntimeWarning for NaNs
         with np.errstate(invalid="ignore"):
-            return left ** right
+            return left**right
 
 
 class Addition(BinaryOperator):
@@ -361,18 +348,18 @@ class Division(BinaryOperator):
         """See :meth:`pybamm.Symbol._diff()`."""
         # apply quotient rule
         top, bottom = self.orphans
-        return (top.diff(variable) * bottom - top * bottom.diff(variable)) / bottom ** 2
+        return (top.diff(variable) * bottom - top * bottom.diff(variable)) / bottom**2
 
     def _binary_jac(self, left_jac, right_jac):
         """See :meth:`pybamm.BinaryOperator._binary_jac()`."""
         # apply quotient rule
         left, right = self.orphans
         if left.evaluates_to_constant_number():
-            return -left / right ** 2 * right_jac
+            return -left / right**2 * right_jac
         elif right.evaluates_to_constant_number():
             return left_jac / right
         else:
-            return (right * left_jac - left * right_jac) / right ** 2
+            return (right * left_jac - left * right_jac) / right**2
 
     def _binary_evaluate(self, left, right):
         """See :meth:`pybamm.BinaryOperator._binary_evaluate()`."""
@@ -581,7 +568,7 @@ class Modulo(BinaryOperator):
         diff = left.diff(variable)
         # derivative if variable is in the right term (rare, check separately to avoid
         # unecessarily big tree)
-        if any(variable.id == x.id for x in right.pre_order()):
+        if any(variable == x for x in right.pre_order()):
             diff += -pybamm.Floor(left / right) * right.diff(variable)
         return diff
 
@@ -636,6 +623,10 @@ class Minimum(BinaryOperator):
         """See :meth:`pybamm.BinaryOperator._binary_new_copy()`."""
         return pybamm.minimum(left, right)
 
+    def _sympy_operator(self, left, right):
+        """Override :meth:`pybamm.BinaryOperator._sympy_operator`"""
+        return sympy.Min(left, right)
+
 
 class Maximum(BinaryOperator):
     """Returns the greater of two objects."""
@@ -668,6 +659,10 @@ class Maximum(BinaryOperator):
         """See :meth:`pybamm.BinaryOperator._binary_new_copy()`."""
         return pybamm.maximum(left, right)
 
+    def _sympy_operator(self, left, right):
+        """Override :meth:`pybamm.BinaryOperator._sympy_operator`"""
+        return sympy.Max(left, right)
+
 
 def simplify_elementwise_binary_broadcasts(left, right):
     left, right = preprocess_binary(left, right)
@@ -770,8 +765,8 @@ def simplified_power(left, right):
         # if (a ** c) is constant or (b ** c) is constant
         if left.left.is_constant() or left.right.is_constant():
             l_left, l_right = left.orphans
-            new_left = l_left ** right
-            new_right = l_right ** right
+            new_left = l_left**right
+            new_right = l_right**right
             if new_left.is_constant() or new_right.is_constant():
                 return new_left * new_right
     elif isinstance(left, Division):
@@ -779,8 +774,8 @@ def simplified_power(left, right):
         # if (a ** c) is constant or (b ** c) is constant
         if left.left.is_constant() or left.right.is_constant():
             l_left, l_right = left.orphans
-            new_left = l_left ** right
-            new_right = l_right ** right
+            new_left = l_left**right
+            new_right = l_right**right
             if new_left.is_constant() or new_right.is_constant():
                 return new_left / new_right
 
@@ -842,7 +837,7 @@ def simplified_addition(left, right):
 
     # Return constant if both sides are constant
     if left.is_constant() and right.is_constant():
-        return pybamm.simplify_if_constant(pybamm.Addition(left, right))
+        return pybamm.simplify_if_constant(Addition(left, right))
 
     # Simplify A @ c + B @ c to (A + B) @ c if (A + B) is constant
     # This is a common construction that appears from discretisation of spatial
@@ -850,17 +845,17 @@ def simplified_addition(left, right):
     elif (
         isinstance(left, MatrixMultiplication)
         and isinstance(right, MatrixMultiplication)
-        and left.right.id == right.right.id
+        and left.right == right.right
     ):
         l_left, l_right = left.orphans
         r_left = right.orphans[0]
         new_left = l_left + r_left
         if new_left.is_constant():
             new_sum = new_left @ l_right
-            new_sum.copy_domains(pybamm.Addition(left, right))
+            new_sum.copy_domains(Addition(left, right))
             return new_sum
 
-    if isinstance(right, pybamm.Addition) and left.is_constant():
+    if isinstance(right, Addition) and left.is_constant():
         # Simplify a + (b + c) to (a + b) + c if (a + b) is constant
         if right.left.is_constant():
             r_left, r_right = right.orphans
@@ -869,7 +864,7 @@ def simplified_addition(left, right):
         elif right.right.is_constant():
             r_left, r_right = right.orphans
             return (left + r_right) + r_left
-    elif isinstance(right, pybamm.Subtraction) and left.is_constant():
+    elif isinstance(right, Subtraction) and left.is_constant():
         # Simplify a + (b - c) to (a + b) - c if (a + b) is constant
         if right.left.is_constant():
             r_left, r_right = right.orphans
@@ -878,7 +873,7 @@ def simplified_addition(left, right):
         elif right.right.is_constant():
             r_left, r_right = right.orphans
             return (left - r_right) + r_left
-    if isinstance(left, pybamm.Addition) and right.is_constant():
+    if isinstance(left, Addition) and right.is_constant():
         # Simplify (a + b) + c to a + (b + c) if (b + c) is constant
         if left.right.is_constant():
             l_left, l_right = left.orphans
@@ -887,7 +882,7 @@ def simplified_addition(left, right):
         elif left.left.is_constant():
             l_left, l_right = left.orphans
             return (l_left + right) + l_right
-    elif isinstance(left, pybamm.Subtraction) and right.is_constant():
+    elif isinstance(left, Subtraction) and right.is_constant():
         # Simplify (a - b) + c to a + (c - b) if (c - b) is constant
         if left.right.is_constant():
             l_left, l_right = left.orphans
@@ -897,7 +892,7 @@ def simplified_addition(left, right):
             l_left, l_right = left.orphans
             return (l_left + right) - l_right
 
-    return pybamm.simplify_if_constant(pybamm.Addition(left, right))
+    return pybamm.simplify_if_constant(Addition(left, right))
 
 
 def simplified_subtraction(left, right):
@@ -955,10 +950,10 @@ def simplified_subtraction(left, right):
         return pybamm.simplify_if_constant(Subtraction(left, right))
 
     # a symbol minus itself is 0s of the same shape
-    if left.id == right.id:
+    if left == right:
         return pybamm.zeros_like(left)
 
-    if isinstance(right, pybamm.Addition) and left.is_constant():
+    if isinstance(right, Addition) and left.is_constant():
         # Simplify a - (b + c) to (a - b) - c if (a - b) is constant
         if right.left.is_constant():
             r_left, r_right = right.orphans
@@ -967,7 +962,7 @@ def simplified_subtraction(left, right):
         elif right.right.is_constant():
             r_left, r_right = right.orphans
             return (left - r_right) - r_left
-    elif isinstance(right, pybamm.Subtraction) and left.is_constant():
+    elif isinstance(right, Subtraction) and left.is_constant():
         # Simplify a - (b - c) to (a - b) + c if (a - b) is constant
         if right.left.is_constant():
             r_left, r_right = right.orphans
@@ -976,7 +971,7 @@ def simplified_subtraction(left, right):
         elif right.right.is_constant():
             r_left, r_right = right.orphans
             return (left + r_right) - r_left
-    if isinstance(left, pybamm.Addition) and right.is_constant():
+    if isinstance(left, Addition) and right.is_constant():
         # Simplify (a + b) - c to a + (b - c) if (b - c) is constant
         if left.right.is_constant():
             l_left, l_right = left.orphans
@@ -985,7 +980,7 @@ def simplified_subtraction(left, right):
         elif left.left.is_constant():
             l_left, l_right = left.orphans
             return (l_left - right) + l_right
-    elif isinstance(left, pybamm.Subtraction) and right.is_constant():
+    elif isinstance(left, Subtraction) and right.is_constant():
         # Simplify (a - b) - c to a - (c + b) if (c + b) is constant
         if left.right.is_constant():
             l_left, l_right = left.orphans
@@ -995,7 +990,7 @@ def simplified_subtraction(left, right):
             l_left, l_right = left.orphans
             return (l_left - right) - l_right
 
-    return pybamm.simplify_if_constant(pybamm.Subtraction(left, right))
+    return pybamm.simplify_if_constant(Subtraction(left, right))
 
 
 def simplified_multiplication(left, right):
@@ -1016,7 +1011,7 @@ def simplified_multiplication(left, right):
 
     # if one of the children is a zero matrix, we have to be careful about shapes
     if pybamm.is_matrix_zero(left) or pybamm.is_matrix_zero(right):
-        return pybamm.zeros_like(pybamm.Multiplication(left, right))
+        return pybamm.zeros_like(Multiplication(left, right))
 
     # anything multiplied by a scalar one returns itself
     if pybamm.is_scalar_one(left):
@@ -1032,7 +1027,7 @@ def simplified_multiplication(left, right):
 
     # Return constant if both sides are constant
     if left.is_constant() and right.is_constant():
-        return pybamm.simplify_if_constant(pybamm.Multiplication(left, right))
+        return pybamm.simplify_if_constant(Multiplication(left, right))
 
     # anything multiplied by a matrix one returns itself if
     # - the shapes are the same
@@ -1126,11 +1121,7 @@ def simplified_multiplication(left, right):
     # operators
     # Also do this for cases like a * (b @ c + d) where (a * b) is constant
     elif isinstance(right, (Addition, Subtraction)):
-        mul_classes = (
-            pybamm.Multiplication,
-            pybamm.MatrixMultiplication,
-            pybamm.Division,
-        )
+        mul_classes = (Multiplication, MatrixMultiplication, Division)
         if (
             right.left.is_constant()
             or right.right.is_constant()
@@ -1157,7 +1148,7 @@ def simplified_multiplication(left, right):
         # Simplify a * (-b) to (-a) * b if (-a) is constant
         return (-left) * right.orphans[0]
 
-    return pybamm.Multiplication(left, right)
+    return Multiplication(left, right)
 
 
 def simplified_division(left, right):
@@ -1174,7 +1165,7 @@ def simplified_division(left, right):
 
     # matrix zero divided by anything returns matrix zero (i.e. itself)
     if pybamm.is_matrix_zero(left):
-        return pybamm.zeros_like(pybamm.Division(left, right))
+        return pybamm.zeros_like(Division(left, right))
 
     # anything divided by zero raises error
     if pybamm.is_scalar_zero(right):
@@ -1185,7 +1176,7 @@ def simplified_division(left, right):
         return left
 
     # a symbol divided by itself is 1s of the same shape
-    if left.id == right.id:
+    if left == right:
         return pybamm.ones_like(left)
 
     # anything multiplied by a matrix one returns itself if
@@ -1209,7 +1200,7 @@ def simplified_division(left, right):
 
     # Return constant if both sides are constant
     if left.is_constant() and right.is_constant():
-        return pybamm.simplify_if_constant(pybamm.Division(left, right))
+        return pybamm.simplify_if_constant(Division(left, right))
 
     # Simplify (B @ c) / a to (B / a) @ c if (B / a) is constant
     # This is a common construction that appears from discretisation of averages
@@ -1262,6 +1253,17 @@ def simplified_division(left, right):
             r_left, r_right = right.orphans
             return (left * r_right) / r_left
 
+    # Cancelling out common terms
+    if isinstance(left, Multiplication) and isinstance(right, Multiplication):
+        if left.left == right.left:
+            _, l_right = left.orphans
+            _, r_right = right.orphans
+            return l_right / r_right
+        if left.right == right.right:
+            l_left, _ = left.orphans
+            r_left, _ = right.orphans
+            return l_left / r_left
+
     # Negation simplifications
     if isinstance(left, pybamm.Negate) and isinstance(right, pybamm.Negate):
         # Double negation cancels out
@@ -1269,17 +1271,18 @@ def simplified_division(left, right):
     elif isinstance(left, pybamm.Negate) and right.is_constant():
         # Simplify (-a) / b to a / (-b) if (-b) is constant
         return left.orphans[0] / (-right)
-    elif isinstance(right, pybamm.Negate) and left.is_constant():
+
+    if isinstance(right, pybamm.Negate) and left.is_constant():
         # Simplify a / (-b) to (-a) / b if (-a) is constant
         return (-left) / right.orphans[0]
 
-    return pybamm.simplify_if_constant(pybamm.Division(left, right))
+    return pybamm.simplify_if_constant(Division(left, right))
 
 
 def simplified_matrix_multiplication(left, right):
     left, right = preprocess_binary(left, right)
     if pybamm.is_matrix_zero(left) or pybamm.is_matrix_zero(right):
-        return pybamm.zeros_like(pybamm.MatrixMultiplication(left, right))
+        return pybamm.zeros_like(MatrixMultiplication(left, right))
 
     if isinstance(right, Multiplication) and left.is_constant():
         # Simplify A @ (b * c) to (A * b) @ c if (A * b) is constant
@@ -1313,18 +1316,35 @@ def simplified_matrix_multiplication(left, right):
         new_mul.copy_domains(right)
         return new_mul
 
-    # Simplify A @ (b + c) to (A @ b) + (A @ c) if (A @ b) or (A @ c) is constant
-    # This is a common construction that appears from discretisation of spatial
-    # operators
-    # Don't do this if either b or c is a number as this will lead to matmul errors
-    elif isinstance(right, Addition):
-        if (right.left.is_constant() or right.right.is_constant()) and not (
+    elif left.is_constant() and isinstance(right, (Addition, Subtraction)):
+        # Simplify A @ (b +- c) to (A @ b) +- (A @ c) if (A @ b) or (A @ c) is constant
+        # This is a common construction that appears from discretisation of spatial
+        # operators
+        # Or simplify A @ (B @ b +- C @ c) to (A @ B @ b) +- (A @ C @ c) if (A @ B)
+        # and (A @ C) are constant
+        # Don't do this if either b or c is a number as this will lead to matmul errors
+        if (
+            (right.left.is_constant() or right.right.is_constant())
+            # these lines should work but don't, possibly because of poorly
+            # conditioned model?
+            # or (
+            #     isinstance(right.left, MatrixMultiplication)
+            #     and right.left.left.is_constant()
+            #     and isinstance(right.right, MatrixMultiplication)
+            #     and right.right.left.is_constant()
+            # )
+        ) and not (
             right.left.size_for_testing == 1 or right.right.size_for_testing == 1
         ):
             r_left, r_right = right.orphans
-            return (left @ r_left) + (left @ r_right)
+            r_left.domains = right.domains
+            r_right.domains = right.domains
+            if isinstance(right, Addition):
+                return (left @ r_left) + (left @ r_right)
+            elif isinstance(right, Subtraction):
+                return (left @ r_left) - (left @ r_right)
 
-    return pybamm.simplify_if_constant(pybamm.MatrixMultiplication(left, right))
+    return pybamm.simplify_if_constant(MatrixMultiplication(left, right))
 
 
 def minimum(left, right):
