@@ -284,7 +284,7 @@ class Symbol:
                 # accounting for empty domains
                 or {k: v for k, v in self._domains.items() if v != []} == domains
             ):
-                return None  # no change
+                return  # no change
         except AttributeError:
             # self._domains has not been set yet
             pass
@@ -293,7 +293,7 @@ class Symbol:
         if domains == {"primary": []}:
             self._domains = EMPTY_DOMAINS
             self.set_id()
-            return None
+            return
 
         # Set default domains
         domains = {**EMPTY_DOMAINS, **domains}
@@ -395,10 +395,6 @@ class Symbol:
         """
         Set the immutable "identity" of a variable (e.g. for identifying y_slices).
 
-        This is identical to what we'd put in a __hash__ function
-        However, implementing __hash__ requires also implementing __eq__,
-        which would then mess with loop-checking in the anytree module.
-
         Hashing can be slow, so we set the id when we create the node, and hence only
         need to hash once.
         """
@@ -408,6 +404,18 @@ class Symbol:
             + tuple([(k, tuple(v)) for k, v in self.domains.items() if v != []])
         )
 
+    def __eq__(self, other):
+        try:
+            return self._id == other._id
+        except AttributeError:
+            if isinstance(other, numbers.Number):
+                return self._id == pybamm.Scalar(other)._id
+            else:
+                return False
+
+    def __hash__(self):
+        return self._id
+
     @property
     def orphans(self):
         """
@@ -648,6 +656,9 @@ class Symbol:
         """return an :class:`Modulo` object."""
         return pybamm.simplify_if_constant(pybamm.Modulo(self, other))
 
+    def __bool__(self):
+        raise NotImplementedError("Boolean operator not defined for Symbols.")
+
     def diff(self, variable):
         """
         Differentiate a symbol with respect to a variable. For any symbol that can be
@@ -660,11 +671,11 @@ class Symbol:
         variable : :class:`pybamm.Symbol`
             The variable with respect to which to differentiate
         """
-        if variable.id == self.id:
+        if variable == self:
             return pybamm.Scalar(1)
-        elif any(variable.id == x.id for x in self.pre_order()):
+        elif any(variable == x for x in self.pre_order()):
             return self._diff(variable)
-        elif variable.id == pybamm.t.id and self.has_symbol_of_classes(
+        elif variable == pybamm.t and self.has_symbol_of_classes(
             (pybamm.VariableBase, pybamm.StateVectorBase)
         ):
             return self._diff(variable)
@@ -724,11 +735,8 @@ class Symbol:
             "{!s} of type {}".format(self, type(self))
         )
 
-    def evaluate(self, t=None, y=None, y_dot=None, inputs=None, known_evals=None):
+    def evaluate(self, t=None, y=None, y_dot=None, inputs=None):
         """Evaluate expression tree (wrapper to allow using dict of known values).
-        If the dict 'known_evals' is provided, the dict is searched for self.id; if
-        self.id is in the keys, return that value; otherwise, evaluate using
-        :meth:`_base_evaluate()` and add that value to known_evals
 
         Parameters
         ----------
@@ -741,22 +749,13 @@ class Symbol:
             (default None)
         inputs : dict, optional
             dictionary of inputs to use when solving (default None)
-        known_evals : dict, optional
-            dictionary containing known values (default None)
 
         Returns
         -------
         number or array
             the node evaluated at (t,y)
-        known_evals (if known_evals input is not None) : dict
-            the dictionary of known values
         """
-        if known_evals is not None:
-            if self.id not in known_evals:
-                known_evals[self.id] = self._base_evaluate(t, y, y_dot, inputs)
-            return known_evals[self.id], known_evals
-        else:
-            return self._base_evaluate(t, y, y_dot, inputs)
+        return self._base_evaluate(t, y, y_dot, inputs)
 
     def evaluate_for_shape(self):
         """
@@ -812,14 +811,16 @@ class Symbol:
                 return None
             elif error.args[0] == "StateVectorDot cannot evaluate input 'y_dot=None'":
                 return None
-            else:
+            else:  # pragma: no cover
                 raise error
-        except ValueError as e:
+        except ValueError as error:
             # return None if specific ValueError is raised
             # (there is a e.g. Time in the tree)
-            if e.args[0] == "t must be provided":
+            if error.args[0] == "t must be provided":
                 return None
-            raise pybamm.ShapeError("Cannot find shape (original error: {})".format(e))
+            raise pybamm.ShapeError(
+                f"Cannot find shape (original error: {error})"
+            )  # pragma: no cover
         return result
 
     def evaluates_to_number(self):
@@ -892,7 +893,7 @@ class Symbol:
         """
         raise NotImplementedError(
             """method self.new_copy() not implemented
-               for symbol {!s} of type {}""".format(
+            for symbol {!s} of type {}""".format(
                 self, type(self)
             )
         )
@@ -932,7 +933,7 @@ class Symbol:
             # If that fails, fall back to calculating how big y should really be
             except ValueError:
                 unpacker = pybamm.SymbolUnpacker(pybamm.StateVector)
-                state_vectors_in_node = unpacker.unpack_symbol(self).values()
+                state_vectors_in_node = unpacker.unpack_symbol(self)
                 min_y_size = max(
                     max(len(x._evaluation_array) for x in state_vectors_in_node), 1
                 )
@@ -994,10 +995,8 @@ class Symbol:
 
     @print_name.setter
     def print_name(self, name):
-        if name is None:
-            self._print_name = name
-        else:
-            self._print_name = prettify_print_name(name)
+        self._raw_print_name = name
+        self._print_name = prettify_print_name(name)
 
     def to_equation(self):
         return sympy.Symbol(str(self.name))
