@@ -20,12 +20,22 @@ class BatteryModelOptions(pybamm.FuzzyDict):
         where a 2-tuple of strings can be provided instead to indicate a different
         option for the negative and positive electrodes.
 
+            * "calculate discharge energy": str
+                Whether to calculate the discharge energy, throughput energy and
+                throughput capacity in addition to discharge capacity. Must be one of
+                "true" or "false". "false" is the default, since calculating discharge
+                energy can be computationally expensive for simple models like SPM.
             * "cell geometry" : str
                 Sets the geometry of the cell. Can be "pouch" (default) or
                 "arbitrary". The arbitrary geometry option solves a 1D electrochemical
                 model with prescribed cell volume and cross-sectional area, and
                 (if thermal effects are included) solves a lumped thermal model
                 with prescribed surface area for cooling.
+            * "calculate heat source for isothermal models" : str
+                Whether to calculate the heat source terms during isothermal operation.
+                Can be "true" or "false". If "false", the heat source terms are set
+                to zero. Default is "false" since this option may require additional
+                parameters not needed by the electrochemical model.
             * "convection" : str
                 Whether to include the effects of convection in the model. Can be
                 "none" (default), "uniform transverse" or "full transverse".
@@ -58,12 +68,20 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                 Can be "full" (default), "constant", or "current-driven".
             * "lithium plating" : str
                 Sets the model for lithium plating. Can be "none" (default),
-                "reversible" or "irreversible".
+                "reversible", "partially reversible", or "irreversible".
+            * "lithium plating porosity change" : str
+                Whether to include porosity change due to lithium plating, can be
+                "false" (default) or "true".
             * "loss of active material" : str
                 Sets the model for loss of active material. Can be "none" (default),
                 "stress-driven", "reaction-driven", or "stress and reaction-driven".
                 A 2-tuple can be provided for different behaviour in negative and
                 positive electrodes.
+            * "particle phases": str
+                Number of phases present in the electrode. A 2-tuple can be provided for
+                different behaviour in negative and positive electrodes.
+                For example, set to ("2", "1") for a negative electrode with 2 phases,
+                e.g. graphite and silicon.
             * "operating mode" : str
                 Sets the operating mode for the model. This determines how the current
                 is set. Can be:
@@ -131,6 +149,9 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                     .. math::
                         \\eta_r = \\frac{F}{RT}
                         * (\\phi_s - \\phi_e - U - R_{sei} * L_{sei} * \\frac{I}{aL})
+            * "SEI on cracks" : str
+                Whether to include SEI growth on particle cracks, can be "false"
+                (default) or "true".
             * "SEI porosity change" : str
                 Whether to include porosity change due to SEI formation, can be "false"
                 (default) or "true".
@@ -157,13 +178,20 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                 Which electrode(s) intercalates and which is counter. If "both"
                 (default), the model is a standard battery. Otherwise can be "negative"
                 or "positive" to indicate a half-cell model.
+            * "x-average side reactions": str
+                Whether to average the side reactions (SEI growth, lithium plating and
+                the respective porosity change) over the x-axis in Single Particle
+                Models, can be "false" or "true". Default is "false" for SPMe and
+                "true" for SPM.
 
     **Extends:** :class:`dict`
     """
 
     def __init__(self, extra_options):
         self.possible_options = {
+            "calculate discharge energy": ["false", "true"],
             "cell geometry": ["arbitrary", "pouch"],
+            "calculate heat source for isothermal models": ["false", "true"],
             "convection": ["none", "uniform transverse", "full transverse"],
             "current collector": [
                 "uniform",
@@ -187,7 +215,12 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                 "Marcus-Hush-Chidsey",
             ],
             "interface utilisation": ["full", "constant", "current-driven"],
-            "lithium plating": ["none", "reversible", "irreversible"],
+            "lithium plating": [
+                "none",
+                "reversible",
+                "partially reversible",
+                "irreversible",
+            ],
             "lithium plating porosity change": ["false", "true"],
             "loss of active material": [
                 "none",
@@ -195,6 +228,7 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                 "reaction-driven",
                 "stress and reaction-driven",
             ],
+            "open circuit potential": ["single", "current sigmoid"],
             "operating mode": [
                 "current",
                 "voltage",
@@ -214,6 +248,7 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                 "quartic profile",
             ],
             "particle mechanics": ["none", "swelling only", "swelling and cracking"],
+            "particle phases": ["1", "2"],
             "particle shape": ["spherical", "no particles"],
             "particle size": ["single", "distribution"],
             "SEI": [
@@ -226,12 +261,14 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                 "ec reaction limited",
             ],
             "SEI film resistance": ["none", "distributed", "average"],
+            "SEI on cracks": ["false", "true"],
             "SEI porosity change": ["false", "true"],
             "stress-induced diffusion": ["false", "true"],
             "surface form": ["false", "differential", "algebraic"],
             "thermal": ["isothermal", "lumped", "x-lumped", "x-full"],
             "total interfacial current density as a state": ["false", "true"],
             "working electrode": ["both", "negative", "positive"],
+            "x-average side reactions": ["false", "true"],
         }
 
         default_options = {
@@ -262,14 +299,27 @@ class BatteryModelOptions(pybamm.FuzzyDict):
         # The "SEI film resistance" option will still be overridden by extra_options if
         # provided
 
-        # Change the default for particle mechanics based on which LAM option is
-        # provided
-        # return "none" if option not given
-        lam_option = extra_options.get("loss of active material", "none")
-        if "stress-driven" in lam_option or "stress and reaction-driven" in lam_option:
-            default_options["particle mechanics"] = "swelling only"
+        # Change the default for particle mechanics based on which SEI on cracks and LAM
+        # options are provided
+        # return "false" and "none" respectively if options not given
+        SEI_cracks_option = extra_options.get("SEI on cracks", "false")
+        LAM_opt = extra_options.get("loss of active material", "none")
+        if SEI_cracks_option == "true":
+            if "stress-driven" in LAM_opt or "stress and reaction-driven" in LAM_opt:
+                default_options["particle mechanics"] = (
+                    "swelling and cracking",
+                    "swelling only",
+                )
+            else:
+                default_options["particle mechanics"] = (
+                    "swelling and cracking",
+                    "none",
+                )
         else:
-            default_options["particle mechanics"] = "none"
+            if "stress-driven" in LAM_opt or "stress and reaction-driven" in LAM_opt:
+                default_options["particle mechanics"] = "swelling only"
+            else:
+                default_options["particle mechanics"] = "none"
         # The "particle mechanics" option will still be overridden by extra_options if
         # provided
 
@@ -289,6 +339,25 @@ class BatteryModelOptions(pybamm.FuzzyDict):
         # The "stress-induced diffusion" option will still be overridden by
         # extra_options if provided
 
+        # Change the default for surface form based on which particle
+        # phases option is provided.
+        # return "1" if option not given
+        phases_option = extra_options.get("particle phases", "1")
+        if phases_option == "1":
+            default_options["surface form"] = "false"
+        else:
+            default_options["surface form"] = "algebraic"
+        # The "surface form" option will still be overridden by
+        # extra_options if provided
+        # Change default SEI model based on which lithium plating option is provided
+        # return "none" if option not given
+        plating_option = extra_options.get("lithium plating", "none")
+        if plating_option == "partially reversible":
+            default_options["SEI"] = "constant"
+        else:
+            default_options["SEI"] = "none"
+        # The "SEI" option will still be overridden by extra_options if provided
+
         options = pybamm.FuzzyDict(default_options)
         # any extra options overwrite the default options
         for name, opt in extra_options.items():
@@ -321,6 +390,24 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                     "current density as a state' must be 'true'"
                 )
 
+        # If "SEI film resistance" is not "none" and there are multiple phases
+        # then "total interfacial current density as a state" must be "true"
+        if (
+            options["SEI film resistance"] != "none"
+            and options["particle phases"] != "1"
+        ):
+            options["total interfacial current density as a state"] = "true"
+            # Check that extra_options did not try to provide a clashing option
+            if (
+                extra_options.get("total interfacial current density as a state")
+                == "false"
+            ):
+                raise pybamm.OptionError(
+                    "If 'SEI film resistance' is not 'none' "
+                    "and there are multiple phases then 'total interfacial "
+                    "current density as a state' must be 'true'"
+                )
+
         # Options not yet compatible with particle-size distributions
         if options["particle size"] == "distribution":
             if options["lithium plating"] != "none":
@@ -415,10 +502,31 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                     f"X-lumped thermal submodels do not yet support {n}D "
                     "current collectors in a half-cell configuration"
                 )
+            elif options["SEI on cracks"] == "true":
+                raise NotImplementedError(
+                    "SEI on cracks not yet implemented for half-cell models"
+                )
+
+        if options["particle phases"] != "1":
+            if not (
+                options["surface form"] != "false"
+                and options["particle size"] == "single"
+                and options["particle"] == "Fickian diffusion"
+                and options["particle mechanics"] == "none"
+                and options["loss of active material"] == "none"
+                and options["lithium plating"] == "none"
+            ):
+                raise pybamm.OptionError(
+                    "If there are multiple particle phases: 'surface form' cannot be "
+                    "'false', 'particle size' must be 'false', 'particle' must be "
+                    "'Fickian diffusion'. Also the following must "
+                    "be 'none': 'particle mechanics', "
+                    "'loss of active material', 'lithium plating'"
+                )
 
         # Check options are valid
         for option, value in options.items():
-            if option == "external submodels" or option == "working electrode":
+            if option in ["external submodels", "working electrode"]:
                 pass
             else:
                 if isinstance(value, str) or option in [
@@ -435,8 +543,10 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                                 "intercalation kinetics",
                                 "interface utilisation",
                                 "loss of active material",
+                                "open circuit potential",
                                 "particle mechanics",
                                 "particle",
+                                "particle phases",
                                 "stress-induced diffusion",
                             ]
                             and isinstance(value, tuple)
@@ -450,7 +560,14 @@ class BatteryModelOptions(pybamm.FuzzyDict):
                             "Values must be strings or (in some cases) "
                             "2-tuples of strings"
                         )
+                # flatten value
+                value_list = []
                 for val in value:
+                    if isinstance(val, tuple):
+                        value_list.extend(list(val))
+                    else:
+                        value_list.append(val)
+                for val in value_list:
                     if option == "timescale":
                         if not (val == "default" or isinstance(val, numbers.Number)):
                             raise pybamm.OptionError(
@@ -466,6 +583,47 @@ class BatteryModelOptions(pybamm.FuzzyDict):
 
         super().__init__(options.items())
 
+    @property
+    def phases(self):
+        try:
+            return self._phases
+        except AttributeError:
+            self._phases = {}
+            for domain in ["negative", "positive"]:
+                number = int(getattr(self, domain)["particle phases"])
+                phases = ["primary"]
+                if number >= 2:
+                    phases.append("secondary")
+                self._phases[domain] = phases
+            return self._phases
+
+    @property
+    def whole_cell_domains(self):
+        try:
+            return self._whole_cell_domains
+        except AttributeError:
+            if self["working electrode"] == "positive":
+                wcd = ["separator", "positive electrode"]
+            elif self["working electrode"] == "negative":
+                wcd = ["negative electrode", "separator"]
+            elif self["working electrode"] == "both":
+                wcd = ["negative electrode", "separator", "positive electrode"]
+            self._whole_cell_domains = wcd
+            return wcd
+
+    @property
+    def electrode_types(self):
+        try:
+            return self._electrode_types
+        except AttributeError:
+            self._electrode_types = {}
+            for domain in ["negative", "positive"]:
+                if f"{domain} electrode" in self.whole_cell_domains:
+                    self._electrode_types[domain] = "porous"
+                else:
+                    self._electrode_types[domain] = "planar"
+            return self._electrode_types
+
     def print_options(self):
         """
         Print the possible options with the ones currently selected
@@ -508,6 +666,29 @@ class BatteryModelDomainOptions(dict):
             # 2-tuple, first is negative domain, second is positive domain
             return options[self.index]
 
+    @property
+    def primary(self):
+        return BatteryModelPhaseOptions(self, 0)
+
+    @property
+    def secondary(self):
+        return BatteryModelPhaseOptions(self, 1)
+
+
+class BatteryModelPhaseOptions(dict):
+    def __init__(self, domain_options, index):
+        super().__init__(domain_options.items())
+        self.domain_options = domain_options
+        self.index = index
+
+    def __getitem__(self, key):
+        options = self.domain_options.__getitem__(key)
+        if isinstance(options, str):
+            return options
+        else:
+            # 2-tuple, first is primary phase, second is secondary phase
+            return options[self.index]
+
 
 class BaseBatteryModel(pybamm.BaseModel):
     """
@@ -518,9 +699,6 @@ class BaseBatteryModel(pybamm.BaseModel):
     def __init__(self, options=None, name="Unnamed battery model"):
         super().__init__(name)
         self.options = options
-        self.submodels = {}
-        self._built = False
-        self._built_fundamental_and_external = False
 
     @pybamm.BaseModel.timescale.setter
     def timescale(self, value):
@@ -552,6 +730,10 @@ class BaseBatteryModel(pybamm.BaseModel):
             "x_p": 20,
             "r_n": 20,
             "r_p": 20,
+            "r_n_prim": 20,
+            "r_p_prim": 20,
+            "r_n_sec": 20,
+            "r_p_sec": 20,
             "y": 10,
             "z": 10,
             "R_n": 30,
@@ -570,6 +752,10 @@ class BaseBatteryModel(pybamm.BaseModel):
             "positive electrode": pybamm.Uniform1DSubMesh,
             "negative particle": pybamm.Uniform1DSubMesh,
             "positive particle": pybamm.Uniform1DSubMesh,
+            "negative primary particle": pybamm.Uniform1DSubMesh,
+            "positive primary particle": pybamm.Uniform1DSubMesh,
+            "negative secondary particle": pybamm.Uniform1DSubMesh,
+            "positive secondary particle": pybamm.Uniform1DSubMesh,
             "negative particle size": pybamm.Uniform1DSubMesh,
             "positive particle size": pybamm.Uniform1DSubMesh,
         }
@@ -588,6 +774,10 @@ class BaseBatteryModel(pybamm.BaseModel):
             "macroscale": pybamm.FiniteVolume(),
             "negative particle": pybamm.FiniteVolume(),
             "positive particle": pybamm.FiniteVolume(),
+            "negative primary particle": pybamm.FiniteVolume(),
+            "positive primary particle": pybamm.FiniteVolume(),
+            "negative secondary particle": pybamm.FiniteVolume(),
+            "positive secondary particle": pybamm.FiniteVolume(),
             "negative particle size": pybamm.FiniteVolume(),
             "positive particle size": pybamm.FiniteVolume(),
         }
@@ -627,6 +817,20 @@ class BaseBatteryModel(pybamm.BaseModel):
                         options["electrolyte conductivity"]
                     )
                 )
+        if isinstance(self, pybamm.lithium_ion.SPM) and not isinstance(
+            self, pybamm.lithium_ion.SPMe
+        ):
+            if options["x-average side reactions"] == "false":
+                raise pybamm.OptionError(
+                    "x-average side reactions cannot be 'false' for SPM models"
+                )
+        if isinstance(self, pybamm.lithium_ion.SPM) and not isinstance(
+            self, pybamm.lithium_ion.MPM
+        ):
+            if options["particle size"] == "distribution":
+                raise pybamm.OptionError(
+                    "'particle size' should be 'single' for SPM and SPMe models"
+                )
         if isinstance(self, pybamm.lead_acid.BaseModel):
             if options["thermal"] != "isothermal" and options["dimensionality"] != 0:
                 raise pybamm.OptionError(
@@ -687,33 +891,6 @@ class BaseBatteryModel(pybamm.BaseModel):
                 {"y": var.y, "y [m]": var.y * L_z, "z": var.z, "z [m]": var.z * L_z}
             )
 
-        # Initialize "total reaction" variables
-        # These will get populated by the "get_coupled_variables" methods, and then used
-        # later by "set_rhs" or "set_algebraic", which ensures that we always have
-        # added all the necessary variables by the time the sum is used
-        self.variables.update(
-            {
-                "Sum of electrolyte reaction source terms": 0,
-                "Sum of positive electrode electrolyte reaction source terms": 0,
-                "Sum of x-averaged positive electrode "
-                "electrolyte reaction source terms": 0,
-                "Sum of interfacial current densities": 0,
-                "Sum of positive electrode interfacial current densities": 0,
-                "Sum of x-averaged positive electrode interfacial current densities": 0,
-            }
-        )
-        if not self.half_cell:
-            self.variables.update(
-                {
-                    "Sum of negative electrode electrolyte reaction source terms": 0,
-                    "Sum of x-averaged negative electrode "
-                    "electrolyte reaction source terms": 0,
-                    "Sum of negative electrode interfacial current densities": 0,
-                    "Sum of x-averaged negative electrode interfacial current densities"
-                    "": 0,
-                }
-            )
-
     def build_fundamental_and_external(self):
         # Get the fundamental variables
         for submodel_name, submodel in self.submodels.items():
@@ -824,22 +1001,10 @@ class BaseBatteryModel(pybamm.BaseModel):
 
     def build_model(self):
 
-        # Check if already built
-        if self._built:
-            raise pybamm.ModelError(
-                """Model already built. If you are adding a new submodel, try using
-                `model.update` instead."""
-            )
-
-        pybamm.logger.info("Start building {}".format(self.name))
-
-        if self._built_fundamental_and_external is False:
-            self.build_fundamental_and_external()
-
-        self.build_coupled_variables()
-
-        self.build_model_equations()
+        # Build model variables and equations
+        self._build_model()
 
+        # Set battery specific variables
         pybamm.logger.debug("Setting voltage variables ({})".format(self.name))
         self.set_voltage_variables()
 
@@ -853,15 +1018,6 @@ class BaseBatteryModel(pybamm.BaseModel):
         self._built = True
         pybamm.logger.info("Finish building {}".format(self.name))
 
-    def new_empty_copy(self):
-        """See :meth:`pybamm.BaseModel.new_empty_copy()`"""
-        new_model = self.__class__(name=self.name, options=self.options)
-        new_model.use_jacobian = self.use_jacobian
-        new_model.convert_to_format = self.convert_to_format
-        new_model._timescale = self.timescale
-        new_model._length_scales = self.length_scales
-        return new_model
-
     @property
     def summary_variables(self):
         return self._summary_variables
@@ -887,7 +1043,7 @@ class BaseBatteryModel(pybamm.BaseModel):
         self._summary_variables = []
 
     def get_intercalation_kinetics(self, domain):
-        options = getattr(self.options, domain.lower())
+        options = getattr(self.options, domain)
         if options["intercalation kinetics"] == "symmetric Butler-Volmer":
             return pybamm.kinetics.SymmetricButlerVolmer
         elif options["intercalation kinetics"] == "asymmetric Butler-Volmer":
@@ -899,8 +1055,7 @@ class BaseBatteryModel(pybamm.BaseModel):
         elif options["intercalation kinetics"] == "Marcus-Hush-Chidsey":
             return pybamm.kinetics.MarcusHushChidsey
 
-    @property
-    def inverse_intercalation_kinetics(self):
+    def get_inverse_intercalation_kinetics(self):
         if self.options["intercalation kinetics"] == "symmetric Butler-Volmer":
             return pybamm.kinetics.InverseButlerVolmer
         else:
@@ -916,78 +1071,75 @@ class BaseBatteryModel(pybamm.BaseModel):
         e.g. (not necessarily constant-) current, voltage, etc
         """
         if self.options["operating mode"] == "current":
-            model = pybamm.external_circuit.ExplicitCurrentControl(self.param)
+            model = pybamm.external_circuit.ExplicitCurrentControl(
+                self.param, self.options
+            )
         elif self.options["operating mode"] == "voltage":
-            model = pybamm.external_circuit.VoltageFunctionControl(self.param)
+            model = pybamm.external_circuit.VoltageFunctionControl(
+                self.param, self.options
+            )
         elif self.options["operating mode"] == "power":
             model = pybamm.external_circuit.PowerFunctionControl(
-                self.param, "algebraic"
+                self.param, self.options, "algebraic"
             )
         elif self.options["operating mode"] == "differential power":
             model = pybamm.external_circuit.PowerFunctionControl(
-                self.param, "differential without max"
+                self.param, self.options, "differential without max"
             )
         elif self.options["operating mode"] == "explicit power":
-            model = pybamm.external_circuit.ExplicitPowerControl(self.param)
+            model = pybamm.external_circuit.ExplicitPowerControl(
+                self.param, self.options
+            )
         elif self.options["operating mode"] == "resistance":
             model = pybamm.external_circuit.ResistanceFunctionControl(
-                self.param, "algebraic"
+                self.param, self.options, "algebraic"
             )
         elif self.options["operating mode"] == "differential resistance":
             model = pybamm.external_circuit.ResistanceFunctionControl(
-                self.param, "differential without max"
+                self.param, self.options, "differential without max"
             )
         elif self.options["operating mode"] == "explicit resistance":
-            model = pybamm.external_circuit.ExplicitResistanceControl(self.param)
+            model = pybamm.external_circuit.ExplicitResistanceControl(
+                self.param, self.options
+            )
         elif self.options["operating mode"] == "CCCV":
-            model = pybamm.external_circuit.CCCVFunctionControl(self.param)
+            model = pybamm.external_circuit.CCCVFunctionControl(
+                self.param, self.options
+            )
         elif callable(self.options["operating mode"]):
             model = pybamm.external_circuit.FunctionControl(
-                self.param, self.options["operating mode"]
+                self.param, self.options["operating mode"], self.options
             )
         self.submodels["external circuit"] = model
 
-    def set_tortuosity_submodels(self):
-        self.submodels["electrolyte tortuosity"] = pybamm.tortuosity.Bruggeman(
+    def set_transport_efficiency_submodels(self):
+        self.submodels[
+            "electrolyte transport efficiency"
+        ] = pybamm.transport_efficiency.Bruggeman(
             self.param, "Electrolyte", self.options
         )
-        self.submodels["electrode tortuosity"] = pybamm.tortuosity.Bruggeman(
-            self.param, "Electrode", self.options
-        )
+        self.submodels[
+            "electrode transport efficiency"
+        ] = pybamm.transport_efficiency.Bruggeman(self.param, "Electrode", self.options)
 
     def set_thermal_submodel(self):
 
         if self.options["thermal"] == "isothermal":
-            thermal_submodel = pybamm.thermal.isothermal.Isothermal(
-                self.param, self.options
-            )
-
+            thermal_submodel = pybamm.thermal.isothermal.Isothermal
         elif self.options["thermal"] == "lumped":
-            thermal_submodel = pybamm.thermal.Lumped(
-                self.param,
-                self.options,
-            )
-
+            thermal_submodel = pybamm.thermal.Lumped
         elif self.options["thermal"] == "x-lumped":
             if self.options["dimensionality"] == 0:
-                thermal_submodel = pybamm.thermal.Lumped(self.param, self.options)
+                thermal_submodel = pybamm.thermal.Lumped
             elif self.options["dimensionality"] == 1:
-                thermal_submodel = pybamm.thermal.pouch_cell.CurrentCollector1D(
-                    self.param,
-                    self.options,
-                )
+                thermal_submodel = pybamm.thermal.pouch_cell.CurrentCollector1D
             elif self.options["dimensionality"] == 2:
-                thermal_submodel = pybamm.thermal.pouch_cell.CurrentCollector2D(
-                    self.param,
-                    self.options,
-                )
+                thermal_submodel = pybamm.thermal.pouch_cell.CurrentCollector2D
         elif self.options["thermal"] == "x-full":
             if self.options["dimensionality"] == 0:
-                thermal_submodel = pybamm.thermal.OneDimensionalX(
-                    self.param, self.options
-                )
+                thermal_submodel = pybamm.thermal.OneDimensionalX
 
-        self.submodels["thermal"] = thermal_submodel
+        self.submodels["thermal"] = thermal_submodel(self.param, self.options)
 
     def set_current_collector_submodel(self):
 
@@ -1001,52 +1153,62 @@ class BaseBatteryModel(pybamm.BaseModel):
         self.submodels["current collector"] = submodel
 
     def set_interface_utilisation_submodel(self):
-        if self.half_cell:
-            domains = ["Counter", "Positive"]
-        else:
-            domains = ["Negative", "Positive"]
-        for domain in domains:
-            name = domain.lower() + " interface utilisation"
-            if domain == "Counter":
-                domain = "Negative"
-            util = getattr(self.options, domain.lower())["interface utilisation"]
+        for domain in ["negative", "positive"]:
+            Domain = domain.capitalize()
+            util = getattr(self.options, domain)["interface utilisation"]
             if util == "full":
-                self.submodels[name] = pybamm.interface_utilisation.Full(
+                submodel = pybamm.interface_utilisation.Full(
                     self.param, domain, self.options
                 )
             elif util == "constant":
-                self.submodels[name] = pybamm.interface_utilisation.Constant(
+                submodel = pybamm.interface_utilisation.Constant(
                     self.param, domain, self.options
                 )
             elif util == "current-driven":
-                if self.half_cell and domain == "Negative":
+                if self.options.electrode_types[domain] == "planar":
                     reaction_loc = "interface"
                 elif self.x_average:
                     reaction_loc = "x-average"
                 else:
                     reaction_loc = "full electrode"
-                self.submodels[name] = pybamm.interface_utilisation.CurrentDriven(
+                submodel = pybamm.interface_utilisation.CurrentDriven(
                     self.param, domain, self.options, reaction_loc
                 )
+            self.submodels[f"{Domain} interface utilisation"] = submodel
 
     def set_voltage_variables(self):
-
-        ocp_n = self.variables["Negative electrode open circuit potential"]
-        ocp_p = self.variables["Positive electrode open circuit potential"]
+        if self.options.negative["particle phases"] == "1":
+            # Only one phase, no need to distinguish between
+            # "primary" and "secondary"
+            phase_n = ""
+        else:
+            # add a space so that we can use "" or (e.g.) "primary " interchangeably
+            # when naming variables
+            phase_n = "primary "
+        if self.options.positive["particle phases"] == "1":
+            phase_p = ""
+        else:
+            phase_p = "primary "
+        ocp_n = self.variables[f"Negative electrode {phase_n}open circuit potential"]
+        ocp_p = self.variables[f"Positive electrode {phase_p}open circuit potential"]
         ocp_n_av = self.variables[
-            "X-averaged negative electrode open circuit potential"
+            f"X-averaged negative electrode {phase_n}open circuit potential"
         ]
         ocp_p_av = self.variables[
-            "X-averaged positive electrode open circuit potential"
+            f"X-averaged positive electrode {phase_p}open circuit potential"
         ]
 
-        ocp_n_dim = self.variables["Negative electrode open circuit potential [V]"]
-        ocp_p_dim = self.variables["Positive electrode open circuit potential [V]"]
+        ocp_n_dim = self.variables[
+            f"Negative electrode {phase_n}open circuit potential [V]"
+        ]
+        ocp_p_dim = self.variables[
+            f"Positive electrode {phase_p}open circuit potential [V]"
+        ]
         ocp_n_av_dim = self.variables[
-            "X-averaged negative electrode open circuit potential [V]"
+            f"X-averaged negative electrode {phase_n}open circuit potential [V]"
         ]
         ocp_p_av_dim = self.variables[
-            "X-averaged positive electrode open circuit potential [V]"
+            f"X-averaged positive electrode {phase_p}open circuit potential [V]"
         ]
 
         ocp_n_left = pybamm.boundary_value(ocp_n, "left")
@@ -1060,7 +1222,7 @@ class BaseBatteryModel(pybamm.BaseModel):
         ocv_dim = ocp_p_right_dim - ocp_n_left_dim
 
         # overpotentials
-        if self.half_cell:
+        if self.options.electrode_types["negative"] == "planar":
             eta_r_n_av = self.variables[
                 "Lithium metal interface reaction overpotential"
             ]
@@ -1069,16 +1231,16 @@ class BaseBatteryModel(pybamm.BaseModel):
             ]
         else:
             eta_r_n_av = self.variables[
-                "X-averaged negative electrode reaction overpotential"
+                f"X-averaged negative electrode {phase_n}reaction overpotential"
             ]
             eta_r_n_av_dim = self.variables[
-                "X-averaged negative electrode reaction overpotential [V]"
+                f"X-averaged negative electrode {phase_n}reaction overpotential [V]"
             ]
         eta_r_p_av = self.variables[
-            "X-averaged positive electrode reaction overpotential"
+            f"X-averaged positive electrode {phase_p}reaction overpotential"
         ]
         eta_r_p_av_dim = self.variables[
-            "X-averaged positive electrode reaction overpotential [V]"
+            f"X-averaged positive electrode {phase_p}reaction overpotential [V]"
         ]
 
         delta_phi_s_n_av = self.variables["X-averaged negative electrode ohmic losses"]
@@ -1097,12 +1259,14 @@ class BaseBatteryModel(pybamm.BaseModel):
         eta_r_av_dim = eta_r_p_av_dim - eta_r_n_av_dim
 
         # SEI film overpotential
-        if self.half_cell:
+        if self.options.electrode_types["negative"] == "planar":
             eta_sei_av = self.variables["SEI film overpotential"]
             eta_sei_av_dim = self.variables["SEI film overpotential [V]"]
         else:
-            eta_sei_av = self.variables["X-averaged SEI film overpotential"]
-            eta_sei_av_dim = self.variables["X-averaged SEI film overpotential [V]"]
+            eta_sei_av = self.variables[f"X-averaged {phase_n}SEI film overpotential"]
+            eta_sei_av_dim = self.variables[
+                f"X-averaged {phase_n}SEI film overpotential [V]"
+            ]
 
         # TODO: add current collector losses to the voltage in 3D
 
@@ -1147,11 +1311,7 @@ class BaseBatteryModel(pybamm.BaseModel):
         # Variables for calculating the equivalent circuit model (ECM) resistance
         # Need to compare OCV to initial value to capture this as an overpotential
         ocv_init = self.param.ocv_init
-        ocv_init_dim = (
-            self.param.U_p_ref
-            - self.param.U_n_ref
-            + self.param.potential_scale * ocv_init
-        )
+        ocv_init_dim = self.param.ocv_ref + self.param.potential_scale * ocv_init
         eta_ocv = ocv - ocv_init
         eta_ocv_dim = ocv_dim - ocv_init_dim
         # Current collector current density for working out euiqvalent resistance
@@ -1197,7 +1357,7 @@ class BaseBatteryModel(pybamm.BaseModel):
         self.events.append(
             pybamm.Event(
                 "Maximum voltage",
-                V - self.param.voltage_high_cut,
+                self.param.voltage_high_cut - V,
                 pybamm.EventType.TERMINATION,
             )
         )
@@ -1246,43 +1406,3 @@ class BaseBatteryModel(pybamm.BaseModel):
         This function is overriden by the base battery models
         """
         pass
-
-    def process_parameters_and_discretise(self, symbol, parameter_values, disc):
-        """
-        Process parameters and discretise a symbol using supplied parameter values
-        and discretisation. Note: care should be taken if using spatial operators
-        on dimensional symbols. Operators in pybamm are written in non-dimensional
-        form, so may need to be scaled by the appropriate length scale. It is
-        recommended to use this method on non-dimensional symbols.
-
-        Parameters
-        ----------
-        symbol : :class:`pybamm.Symbol`
-            Symbol to be processed
-        parameter_values : :class:`pybamm.ParameterValues`
-            The parameter values to use during processing
-        disc : :class:`pybamm.Discretisation`
-            The discrisation to use
-
-        Returns
-        -------
-        :class:`pybamm.Symbol`
-            Processed symbol
-        """
-        # Set y slices
-        if disc.y_slices == {}:
-            variables = list(self.rhs.keys()) + list(self.algebraic.keys())
-            disc.set_variable_slices(variables)
-
-        # Set boundary conditions (also requires setting parameter values)
-        if disc.bcs == {}:
-            self.boundary_conditions = parameter_values.process_boundary_conditions(
-                self
-            )
-            disc.bcs = disc.process_boundary_conditions(self)
-
-        # Process
-        param_symbol = parameter_values.process_symbol(symbol)
-        disc_symbol = disc.process_symbol(param_symbol)
-
-        return disc_symbol
