@@ -19,7 +19,6 @@ class BaseModel(pybamm.BaseBatteryModel):
         options = options or {}
         # Specify that there are no particles in lead-acid, and no half-cell models
         options["particle shape"] = "no particles"
-        self.half_cell = False
         super().__init__(options, name)
         self.param = pybamm.LeadAcidParameters()
 
@@ -81,18 +80,36 @@ class BaseModel(pybamm.BaseBatteryModel):
             self.rhs[fci] = -self.variables["Total current density"] * 100
             self.initial_conditions[fci] = self.param.q_init * 100
 
+    def set_open_circuit_potential_submodel(self):
+        for domain in ["negative", "positive"]:
+            self.submodels[
+                f"{domain} open circuit potential"
+            ] = pybamm.open_circuit_potential.SingleOpenCircuitPotential(
+                self.param, domain, "lead-acid main", self.options, "primary"
+            )
+            self.submodels[
+                f"{domain} oxygen open circuit potential"
+            ] = pybamm.open_circuit_potential.SingleOpenCircuitPotential(
+                self.param, domain, "lead-acid oxygen", self.options, "primary"
+            )
+
     def set_active_material_submodel(self):
-        self.submodels["negative active material"] = pybamm.active_material.Constant(
-            self.param, "Negative", self.options
-        )
-        self.submodels["positive active material"] = pybamm.active_material.Constant(
-            self.param, "Positive", self.options
-        )
+        for domain in ["negative", "positive"]:
+            self.submodels[
+                f"{domain} active material"
+            ] = pybamm.active_material.Constant(
+                self.param, domain, self.options, "primary"
+            )
 
     def set_sei_submodel(self):
 
-        self.submodels["sei"] = pybamm.sei.NoSEI(self.param)
+        self.submodels["sei"] = pybamm.sei.NoSEI(self.param, self.options)
 
     def set_lithium_plating_submodel(self):
 
         self.submodels["lithium plating"] = pybamm.lithium_plating.NoPlating(self.param)
+
+    def set_total_interface_submodel(self):
+        self.submodels["total interface"] = pybamm.interface.TotalInterfacialCurrent(
+            self.param, "lead-acid", self.options
+        )
