@@ -2,7 +2,13 @@
 # Root of the lithium-ion models module.
 #
 from .base_lithium_ion_model import BaseModel
-from .electrode_soh import ElectrodeSOH, get_initial_stoichiometries
+from .electrode_soh import (
+    ElectrodeSOH,
+    ElectrodeSOHx100,
+    ElectrodeSOHx0,
+    ElectrodeSOHSolver,
+    get_initial_stoichiometries,
+)
 from .electrode_soh_half_cell import ElectrodeSOHHalfCell
 from .spm import SPM
 from .spme import SPMe
@@ -11,5 +17,6 @@ from .newman_tobias import NewmanTobias
 from .basic_dfn import BasicDFN
 from .basic_spm import BasicSPM
 from .basic_dfn_half_cell import BasicDFNHalfCell
+from .basic_dfn_composite import BasicDFNComposite
 from .Yang2017 import Yang2017
-from .mpm import MPM
\ No newline at end of file
+from .mpm import MPM
