@@ -27,37 +27,36 @@ class Composite(BaseModel):
         super().__init__(param, domain, options=options)
 
     def get_coupled_variables(self, variables):
+        domain = self.domain
         param = self.param
 
         i_boundary_cc_0 = variables["Leading-order current collector current density"]
 
         # import parameters and spatial variables
-        l_n = param.l_n
-        l_p = param.l_p
+        l_n = param.n.l
+        l_p = param.p.l
         x_n = pybamm.standard_spatial_vars.x_n
         x_p = pybamm.standard_spatial_vars.x_p
 
         tor_0 = variables[
-            "Leading-order x-averaged " + self.domain.lower() + " electrode tortuosity"
+            f"Leading-order x-averaged {domain} electrode transport efficiency"
         ]
         phi_s_cn = variables["Negative current collector potential"]
-        T = variables["X-averaged " + self.domain.lower() + " electrode temperature"]
+        T = variables[f"X-averaged {domain} electrode temperature"]
 
-        if self._domain == "Negative":
-            sigma_eff_0 = self.param.sigma_n(T) * tor_0
+        sigma_eff_0 = self.domain_param.sigma(T) * tor_0
+        if self._domain == "negative":
             phi_s = phi_s_cn + (i_boundary_cc_0 / sigma_eff_0) * (
                 x_n * (x_n - 2 * l_n) / (2 * l_n)
             )
             i_s = i_boundary_cc_0 * (1 - x_n / l_n)
 
-        elif self.domain == "Positive":
+        elif self.domain == "positive":
             delta_phi_p_av = variables[
                 "X-averaged positive electrode surface potential difference"
             ]
             phi_e_p_av = variables["X-averaged positive electrolyte potential"]
 
-            sigma_eff_0 = self.param.sigma_p(T) * tor_0
-
             const = (
                 delta_phi_p_av
                 + phi_e_p_av
@@ -72,27 +71,28 @@ class Composite(BaseModel):
         variables.update(self._get_standard_potential_variables(phi_s))
         variables.update(self._get_standard_current_variables(i_s))
 
-        if self.domain == "Positive":
+        if self.domain == "positive":
             variables.update(self._get_standard_whole_cell_variables(variables))
 
         return variables
 
     def set_boundary_conditions(self, variables):
+        domain, Domain = self.domain_Domain
 
-        phi_s = variables[self.domain + " electrode potential"]
+        phi_s = variables[f"{Domain} electrode potential"]
         tor_0 = variables[
-            "Leading-order x-averaged " + self.domain.lower() + " electrode tortuosity"
+            f"Leading-order x-averaged {domain} electrode transport efficiency"
         ]
         i_boundary_cc_0 = variables["Leading-order current collector current density"]
-        T = variables["X-averaged " + self.domain.lower() + " electrode temperature"]
+        T = variables[f"X-averaged {domain} electrode temperature"]
 
-        if self.domain == "Negative":
+        if self.domain == "negative":
             lbc = (pybamm.Scalar(0), "Dirichlet")
             rbc = (pybamm.Scalar(0), "Neumann")
 
-        elif self.domain == "Positive":
+        elif self.domain == "positive":
             lbc = (pybamm.Scalar(0), "Neumann")
-            sigma_eff_0 = self.param.sigma_p(T) * tor_0
+            sigma_eff_0 = self.param.p.sigma(T) * tor_0
             rbc = (-i_boundary_cc_0 / sigma_eff_0, "Neumann")
 
         self.boundary_conditions[phi_s] = {"left": lbc, "right": rbc}
