@@ -29,9 +29,9 @@ class BaseLeadingOrderSurfaceForm(LeadingOrder):
 
     def get_fundamental_variables(self):
 
-        if self.domain == "Negative":
+        if self.domain == "negative":
             delta_phi_av = pybamm.standard_variables.delta_phi_n_av
-        elif self.domain == "Positive":
+        elif self.domain == "positive":
             delta_phi_av = pybamm.standard_variables.delta_phi_p_av
 
         variables = self._get_standard_average_surface_potential_difference_variables(
@@ -44,27 +44,22 @@ class BaseLeadingOrderSurfaceForm(LeadingOrder):
 
     def get_coupled_variables(self, variables):
         # Only update coupled variables once
-        if self.domain == "Negative":
+        if self.domain == "negative":
             return super().get_coupled_variables(variables)
         else:
             return variables
 
     def set_initial_conditions(self, variables):
-
+        domain = self.domain
         delta_phi = variables[
-            "X-averaged "
-            + self.domain.lower()
-            + " electrode surface potential difference"
+            f"X-averaged {domain} electrode surface potential difference"
         ]
-        if self.domain == "Negative":
-            delta_phi_init = self.param.U_n(self.param.c_n_init(0), self.param.T_init)
-        elif self.domain == "Positive":
-            delta_phi_init = self.param.U_p(self.param.c_p_init(1), self.param.T_init)
+        delta_phi_init = self.domain_param.prim.U_init
 
         self.initial_conditions = {delta_phi: delta_phi_init}
 
     def set_boundary_conditions(self, variables):
-        if self.domain == "Negative":
+        if self.domain == "negative":
             phi_e = variables["Electrolyte potential"]
             self.boundary_conditions = {
                 phi_e: {
@@ -94,31 +89,23 @@ class LeadingOrderDifferential(BaseLeadingOrderSurfaceForm):
         super().__init__(param, domain, options)
 
     def set_rhs(self, variables):
-        param = self.param
-
-        sum_j = variables[
-            "Sum of x-averaged "
-            + self.domain.lower()
-            + " electrode interfacial current densities"
+        domain = self.domain
+        sum_a_j = variables[
+            f"Sum of x-averaged {domain} electrode volumetric "
+            "interfacial current densities"
         ]
 
-        sum_j_av = variables[
-            "X-averaged "
-            + self.domain.lower()
-            + " electrode total interfacial current density"
+        sum_a_j_av = variables[
+            f"X-averaged {domain} electrode total volumetric "
+            "interfacial current density"
         ]
         delta_phi = variables[
-            "X-averaged "
-            + self.domain.lower()
-            + " electrode surface potential difference"
+            f"X-averaged {domain} electrode surface potential difference"
         ]
 
-        if self.domain == "Negative":
-            C_dl = param.C_dl_n
-        elif self.domain == "Positive":
-            C_dl = param.C_dl_p
+        C_dl = self.domain_param.C_dl
 
-        self.rhs[delta_phi] = 1 / C_dl * (sum_j_av - sum_j)
+        self.rhs[delta_phi] = 1 / C_dl * (sum_a_j_av - sum_a_j)
 
 
 class LeadingOrderAlgebraic(BaseLeadingOrderSurfaceForm):
@@ -141,21 +128,18 @@ class LeadingOrderAlgebraic(BaseLeadingOrderSurfaceForm):
         super().__init__(param, domain, options)
 
     def set_algebraic(self, variables):
-        sum_j = variables[
-            "Sum of x-averaged "
-            + self.domain.lower()
-            + " electrode interfacial current densities"
+        domain = self.domain
+        sum_a_j = variables[
+            f"Sum of x-averaged {domain} electrode volumetric "
+            "interfacial current densities"
         ]
 
-        sum_j_av = variables[
-            "X-averaged "
-            + self.domain.lower()
-            + " electrode total interfacial current density"
+        sum_a_j_av = variables[
+            f"X-averaged {domain} electrode total volumetric "
+            "interfacial current density"
         ]
         delta_phi = variables[
-            "X-averaged "
-            + self.domain.lower()
-            + " electrode surface potential difference"
+            f"X-averaged {domain} electrode surface potential difference"
         ]
 
-        self.algebraic[delta_phi] = sum_j_av - sum_j
+        self.algebraic[delta_phi] = sum_a_j_av - sum_a_j
