@@ -1,9 +1,15 @@
 from .base_external_circuit import BaseModel, LeadingOrderBaseModel
-from .current_control_external_circuit import CurrentControl, LeadingOrderCurrentControl
+from .explicit_control_external_circuit import (
+    ExplicitCurrentControl,
+    ExplicitPowerControl,
+    ExplicitResistanceControl,
+    LeadingOrderExplicitCurrentControl,
+)
 from .function_control_external_circuit import (
     FunctionControl,
     VoltageFunctionControl,
     PowerFunctionControl,
+    ResistanceFunctionControl,
     CCCVFunctionControl,
     LeadingOrderFunctionControl,
     LeadingOrderVoltageFunctionControl,
