@@ -18,8 +18,7 @@ class FirstOrder(BaseModel):
     ----------
     param : parameter class
         The parameters to use for this submodel
-    reactions : dict
-        Dictionary of reaction terms
+
 
     **Extends:** :class:`pybamm.oxygen_diffusion.BaseModel`
     """
@@ -30,16 +29,18 @@ class FirstOrder(BaseModel):
     def get_coupled_variables(self, variables):
 
         param = self.param
-        l_n = param.l_n
-        l_s = param.l_s
-        l_p = param.l_p
+        l_n = param.n.l
+        l_s = param.s.l
+        l_p = param.p.l
         x_s = pybamm.standard_spatial_vars.x_s
         x_p = pybamm.standard_spatial_vars.x_p
 
         # Unpack
-        tor_s_0_av = variables["Leading-order x-averaged separator tortuosity"]
+        tor_s_0_av = variables[
+            "Leading-order x-averaged separator electrolyte transport efficiency"
+        ]
         tor_p_0_av = variables[
-            "Leading-order x-averaged positive electrolyte tortuosity"
+            "Leading-order x-averaged positive electrolyte transport efficiency"
         ]
 
         # Diffusivities
@@ -62,17 +63,18 @@ class FirstOrder(BaseModel):
         c_ox_n_1 = pybamm.FullBroadcast(0, "negative electrode", "current collector")
         c_ox_s_1 = sj_ox_p * l_p / D_ox_s * (x_s - l_n)
         c_ox_p_1 = (
-            -sj_ox_p / (2 * D_ox_p) * ((x_p - 1) ** 2 - l_p ** 2)
+            -sj_ox_p / (2 * D_ox_p) * ((x_p - 1) ** 2 - l_p**2)
             + sj_ox_p * l_p * l_s / D_ox_s
         )
 
         # Update variables
-        c_ox = pybamm.Concatenation(
-            param.C_e * c_ox_n_1, param.C_e * c_ox_s_1, param.C_e * c_ox_p_1
+        variables.update(
+            self._get_standard_concentration_variables(
+                param.C_e * c_ox_n_1, param.C_e * c_ox_s_1, param.C_e * c_ox_p_1
+            )
         )
-        variables.update(self._get_standard_concentration_variables(c_ox))
 
-        N_ox = pybamm.Concatenation(
+        N_ox = pybamm.concatenation(
             param.C_e * N_ox_n_1, param.C_e * N_ox_s_1, param.C_e * N_ox_p_1
         )
         variables.update(self._get_standard_flux_variables(N_ox))
