@@ -14,8 +14,7 @@ class LeadingOrder(BaseModel):
     ----------
     param : parameter class
         The parameters to use for this submodel
-    reactions : dict
-        Dictionary of reaction terms
+
 
     **Extends:** :class:`pybamm.oxgen_diffusion.BaseModel`
     """
@@ -24,17 +23,14 @@ class LeadingOrder(BaseModel):
         super().__init__(param)
 
     def get_fundamental_variables(self):
-        c_ox_av = pybamm.Variable("X-averaged oxygen concentration")
-        c_ox_n = pybamm.FullBroadcast(
-            c_ox_av, ["negative electrode"], "current collector"
+        c_ox_av = pybamm.Variable(
+            "X-averaged oxygen concentration", domain="current collector"
         )
-        c_ox_s = pybamm.FullBroadcast(c_ox_av, ["separator"], "current collector")
-        c_ox_p = pybamm.FullBroadcast(
-            c_ox_av, ["positive electrode"], "current collector"
-        )
-        c_ox = pybamm.Concatenation(c_ox_n, c_ox_s, c_ox_p)
+        c_ox_n = pybamm.PrimaryBroadcast(c_ox_av, "negative electrode")
+        c_ox_s = pybamm.PrimaryBroadcast(c_ox_av, "separator")
+        c_ox_p = pybamm.PrimaryBroadcast(c_ox_av, "positive electrode")
 
-        return self._get_standard_concentration_variables(c_ox)
+        return self._get_standard_concentration_variables(c_ox_n, c_ox_s, c_ox_p)
 
     def get_coupled_variables(self, variables):
 
@@ -69,16 +65,16 @@ class LeadingOrder(BaseModel):
         ]
 
         source_terms = (
-            param.l_n * param.s_ox_Ox * j_ox_n_av
-            + param.l_p * param.s_ox_Ox * j_ox_p_av
+            param.n.l * param.s_ox_Ox * j_ox_n_av
+            + param.p.l * param.s_ox_Ox * j_ox_p_av
         )
 
         self.rhs = {
             c_ox_av: 1
-            / (param.l_n * eps_n_av + param.l_s * eps_s_av + param.l_p * eps_p_av)
+            / (param.n.l * eps_n_av + param.s.l * eps_s_av + param.p.l * eps_p_av)
             * (
                 source_terms
-                - c_ox_av * (param.l_n * deps_n_dt_av + param.l_p * deps_p_dt_av)
+                - c_ox_av * (param.n.l * deps_n_dt_av + param.p.l * deps_p_dt_av)
             )
         }
 
