@@ -6,5 +6,5 @@ from pybamm import Scalar
 
 R = Scalar(constants.R)
 F = Scalar(constants.physical_constants["Faraday constant"][0])
-k_b = constants.physical_constants["Boltzmann constant"][0]
-q_e = constants.physical_constants["electron volt"][0]
+k_b = Scalar(constants.physical_constants["Boltzmann constant"][0])
+q_e = Scalar(constants.physical_constants["electron volt"][0])
