@@ -36,7 +36,7 @@ class AlgebraicSolver(pybamm.BaseSolver):
         self.extra_options = extra_options or {}
         self.name = "Algebraic solver ({})".format(method)
         self.algebraic_solver = True
-        pybamm.citations.register("virtanen2020scipy")
+        pybamm.citations.register("Virtanen2020")
 
     @property
     def tol(self):
@@ -46,7 +46,7 @@ class AlgebraicSolver(pybamm.BaseSolver):
     def tol(self, value):
         self._tol = value
 
-    def _integrate(self, model, t_eval, inputs=None):
+    def _integrate(self, model, t_eval, inputs_dict=None):
         """
         Calculate the solution of the algebraic equations through root-finding
 
@@ -56,16 +56,19 @@ class AlgebraicSolver(pybamm.BaseSolver):
             The model whose solution to calculate.
         t_eval : :class:`numpy.array`, size (k,)
             The times at which to compute the solution
-        inputs : dict, optional
+        inputs_dict : dict, optional
             Any input parameters to pass to the model when solving
         """
-        inputs = inputs or {}
+        inputs_dict = inputs_dict or {}
         if model.convert_to_format == "casadi":
-            inputs = casadi.vertcat(*[x for x in inputs.values()])
+            inputs = casadi.vertcat(*[x for x in inputs_dict.values()])
+        else:
+            inputs = inputs_dict
 
         y0 = model.y0
         if isinstance(y0, casadi.DM):
-            y0 = y0.full().flatten()
+            y0 = y0.full()
+        y0 = y0.flatten()
 
         # The casadi algebraic solver can read rhs equations, but leaves them unchanged
         # i.e. the part of the solution vector that corresponds to the differential
@@ -78,16 +81,30 @@ class AlgebraicSolver(pybamm.BaseSolver):
             len_rhs = model.rhs_eval(t_eval[0], y0, inputs).shape[0]
         y0_diff, y0_alg = np.split(y0, [len_rhs])
 
-        algebraic = model.algebraic_eval
+        test_result = model.algebraic_eval(0, y0, inputs)
+
+        if isinstance(test_result, casadi.DM):
+
+            def algebraic(t, y):
+                result = model.algebraic_eval(t, y, inputs)
+                return result.full().flatten()
+
+        else:
+
+            def algebraic(t, y):
+                result = model.algebraic_eval(t, y, inputs)
+                return result.flatten()
 
         y_alg = np.empty((len(y0_alg), len(t_eval)))
 
+        timer = pybamm.Timer()
+        integration_time = 0
         for idx, t in enumerate(t_eval):
 
             def root_fun(y_alg):
                 "Evaluates algebraic using y"
                 y = np.concatenate([y0_diff, y_alg])
-                out = algebraic(t, y, inputs)
+                out = algebraic(t, y)
                 pybamm.logger.debug(
                     "Evaluating algebraic equations at t={}, L2-norm is {}".format(
                         t * model.timescale_eval, np.linalg.norm(out)
@@ -118,15 +135,12 @@ class AlgebraicSolver(pybamm.BaseSolver):
             else:
                 jac_fn = None
 
-            # Evaluate algebraic with new t and previous y0, if it's already close
-            # enough then keep it
-            if np.all(abs(algebraic(t, y0, inputs)) < self.tol):
-                pybamm.logger.debug("Keeping same solution at t={}".format(t))
-                y_alg[:, idx] = y0_alg
-            # Otherwise calculate new y0
-            else:
-                # Methods which use least-squares are specified as either "lsq", which
-                # uses the default method, or with "lsq__methodname"
+            itr = 0
+            maxiter = 2
+            success = False
+            while not success:
+                # Methods which use least-squares are specified as either "lsq",
+                # which uses the default method, or with "lsq__methodname"
                 if self.method.startswith("lsq"):
 
                     if self.method == "lsq":
@@ -135,6 +149,7 @@ class AlgebraicSolver(pybamm.BaseSolver):
                         method = self.method[5:]
                     if jac_fn is None:
                         jac_fn = "2-point"
+                    timer.reset()
                     sol = optimize.least_squares(
                         root_fun,
                         y0_alg,
@@ -144,8 +159,9 @@ class AlgebraicSolver(pybamm.BaseSolver):
                         bounds=model.bounds,
                         **self.extra_options,
                     )
-                # Methods which use minimize are specified as either "minimize", which
-                # uses the default method, or with "minimize__methodname"
+                    integration_time += timer.time()
+                # Methods which use minimize are specified as either "minimize",
+                # which uses the default method, or with "minimize__methodname"
                 elif self.method.startswith("minimize"):
                     # Adapt the root function for minimize
                     def root_norm(y):
@@ -170,6 +186,7 @@ class AlgebraicSolver(pybamm.BaseSolver):
                             (lb, ub) for lb, ub in zip(model.bounds[0], model.bounds[1])
                         ]
                         extra_options["bounds"] = bounds
+                    timer.reset()
                     sol = optimize.minimize(
                         root_norm,
                         y0_alg,
@@ -178,7 +195,9 @@ class AlgebraicSolver(pybamm.BaseSolver):
                         jac=jac_norm,
                         **extra_options,
                     )
+                    integration_time += timer.time()
                 else:
+                    timer.reset()
                     sol = optimize.root(
                         root_fun,
                         y0_alg,
@@ -187,27 +206,34 @@ class AlgebraicSolver(pybamm.BaseSolver):
                         jac=jac_fn,
                         options=self.extra_options,
                     )
+                    integration_time += timer.time()
 
                 if sol.success and np.all(abs(sol.fun) < self.tol):
                     # update initial guess for the next iteration
                     y0_alg = sol.x
                     # update solution array
                     y_alg[:, idx] = y0_alg
+                    success = True
                 elif not sol.success:
                     raise pybamm.SolverError(
                         "Could not find acceptable solution: {}".format(sol.message)
                     )
                 else:
-                    raise pybamm.SolverError(
-                        "Could not find acceptable solution: solver terminated "
-                        "successfully, but maximum solution error "
-                        "({}) above tolerance ({})".format(
-                            np.max(abs(sol.fun)), self.tol
+                    y0_alg = sol.x
+                    if itr > maxiter:
+                        raise pybamm.SolverError(
+                            "Could not find acceptable solution: solver terminated "
+                            "successfully, but maximum solution error "
+                            "({}) above tolerance ({})".format(
+                                np.max(abs(sol.fun)), self.tol
+                            )
                         )
-                    )
+                itr += 1
 
         # Concatenate differential part
         y_diff = np.r_[[y0_diff] * len(t_eval)].T
         y_sol = np.r_[y_diff, y_alg]
         # Return solution object (no events, so pass None to t_event, y_event)
-        return pybamm.Solution(t_eval, y_sol, termination="success")
+        sol = pybamm.Solution(t_eval, y_sol, model, inputs_dict, termination="success")
+        sol.integration_time = integration_time
+        return sol
