@@ -33,7 +33,14 @@ class ProcessedVariable(object):
         Default is True.
     """
 
-    def __init__(self, base_variables, base_variables_casadi, solution, warn=True):
+    def __init__(
+        self,
+        base_variables,
+        base_variables_casadi,
+        solution,
+        warn=True,
+        cumtrapz_ic=None,
+    ):
         self.base_variables = base_variables
         self.base_variables_casadi = base_variables_casadi
 
@@ -44,10 +51,9 @@ class ProcessedVariable(object):
 
         self.mesh = base_variables[0].mesh
         self.domain = base_variables[0].domain
-        self.auxiliary_domains = base_variables[0].auxiliary_domains
+        self.domains = base_variables[0].domains
         self.warn = warn
-
-        self.symbolic_inputs = solution.has_symbolic_inputs
+        self.cumtrapz_ic = cumtrapz_ic
 
         # Sensitivity starts off uninitialized, only set when called
         self._sensitivities = None
@@ -108,6 +114,7 @@ class ProcessedVariable(object):
         # initialise empty array of the correct size
         entries = np.empty(len(self.t_pts))
         idx = 0
+        last_t = 0
         # Evaluate the base_variable index-by-index
         for ts, ys, inputs, base_var_casadi in zip(
             self.all_ts, self.all_ys, self.all_inputs_casadi, self.base_variables_casadi
@@ -115,8 +122,22 @@ class ProcessedVariable(object):
             for inner_idx, t in enumerate(ts):
                 t = ts[inner_idx]
                 y = ys[:, inner_idx]
-                entries[idx] = base_var_casadi(t, y, inputs).full()[0, 0]
+                if self.cumtrapz_ic is not None:
+                    if idx == 0:
+                        new_val = t * base_var_casadi(t, y, inputs).full()[0, 0]
+                        entries[idx] = self.cumtrapz_ic + (
+                            t * base_var_casadi(t, y, inputs).full()[0, 0]
+                        )
+                    else:
+                        new_val = (t - last_t) * (
+                            base_var_casadi(t, y, inputs).full()[0, 0]
+                        )
+                        entries[idx] = new_val + entries[idx - 1]
+                else:
+                    entries[idx] = base_var_casadi(t, y, inputs).full()[0, 0]
+
                 idx += 1
+                last_t = t
 
         # set up interpolation
         if len(self.t_pts) == 1:
@@ -171,7 +192,7 @@ class ProcessedVariable(object):
         # assign attributes for reference (either x_sol or r_sol)
         self.entries = entries
         self.dimensions = 1
-        if self.domain[0] in ["negative particle", "positive particle"]:
+        if self.domain[0].endswith("particle"):
             self.first_dimension = "r"
             self.r_sol = space
         elif self.domain[0] in [
@@ -184,10 +205,7 @@ class ProcessedVariable(object):
         elif self.domain == ["current collector"]:
             self.first_dimension = "z"
             self.z_sol = space
-        elif self.domain[0] in [
-            "negative particle size",
-            "positive particle size",
-        ]:
+        elif self.domain[0].endswith("particle size"):
             self.first_dimension = "R"
             self.R_sol = space
         else:
@@ -303,66 +321,46 @@ class ProcessedVariable(object):
         )
 
         # Process r-x, x-z, r-R, R-x, or R-z
-        if self.domain[0] in [
-            "negative particle",
-            "positive particle",
-            "working particle",
-        ] and self.auxiliary_domains["secondary"][0] in [
-            "negative electrode",
-            "positive electrode",
-            "working electrode",
-        ]:
+        if self.domain[0].endswith("particle") and self.domains["secondary"][
+            0
+        ].endswith("electrode"):
             self.first_dimension = "r"
             self.second_dimension = "x"
             self.r_sol = first_dim_pts
             self.x_sol = second_dim_pts
-        elif (
-            self.domain[0]
-            in [
-                "negative electrode",
-                "separator",
-                "positive electrode",
-            ]
-            and self.auxiliary_domains["secondary"] == ["current collector"]
-        ):
+        elif self.domain[0] in [
+            "negative electrode",
+            "separator",
+            "positive electrode",
+        ] and self.domains["secondary"] == ["current collector"]:
             self.first_dimension = "x"
             self.second_dimension = "z"
             self.x_sol = first_dim_pts
             self.z_sol = second_dim_pts
-        elif self.domain[0] in [
-            "negative particle",
-            "positive particle",
-        ] and self.auxiliary_domains["secondary"][0] in [
-            "negative particle size",
-            "positive particle size",
-        ]:
+        elif self.domain[0].endswith("particle") and self.domains["secondary"][
+            0
+        ].endswith("particle size"):
             self.first_dimension = "r"
             self.second_dimension = "R"
             self.r_sol = first_dim_pts
             self.R_sol = second_dim_pts
-        elif self.domain[0] in [
-            "negative particle size",
-            "positive particle size",
-        ] and self.auxiliary_domains["secondary"][0] in [
-            "negative electrode",
-            "positive electrode",
-        ]:
+        elif self.domain[0].endswith("particle size") and self.domains["secondary"][
+            0
+        ].endswith("electrode"):
             self.first_dimension = "R"
             self.second_dimension = "x"
             self.R_sol = first_dim_pts
             self.x_sol = second_dim_pts
-        elif self.domain[0] in [
-            "negative particle size",
-            "positive particle size",
-        ] and self.auxiliary_domains["secondary"] == ["current collector"]:
+        elif self.domain[0].endswith("particle size") and self.domains["secondary"] == [
+            "current collector"
+        ]:
             self.first_dimension = "R"
             self.second_dimension = "z"
             self.R_sol = first_dim_pts
             self.z_sol = second_dim_pts
-        else:
+        else:  # pragma: no cover
             raise pybamm.DomainError(
-                "Cannot process 3D object with domain '{}' "
-                "and auxiliary_domains '{}'".format(self.domain, self.auxiliary_domains)
+                f"Cannot process 2D object with domains '{self.domains}'."
             )
 
         # assign attributes for reference
@@ -379,7 +377,7 @@ class ProcessedVariable(object):
         first_dim_pts_for_interp = first_dim_pts * first_length_scale
 
         second_length_scale = self.get_spatial_scale(
-            self.second_dimension, self.auxiliary_domains["secondary"][0]
+            self.second_dimension, self.domains["secondary"][0]
         )
         second_dim_pts_for_interp = second_dim_pts * second_length_scale
 
@@ -563,6 +561,7 @@ class ProcessedVariable(object):
             name: casadi.MX.sym(name, value.shape[0])
             for name, value in self.all_inputs[0].items()
         }
+
         p_casadi_stacked = casadi.vertcat(*[p for p in p_casadi.values()])
 
         # Convert variable to casadi format for differentiating
