@@ -73,6 +73,7 @@ class Solution(object):
         y_event=None,
         termination="final time",
         sensitivities=False,
+        check_solution=True,
     ):
         if not isinstance(all_ts, list):
             all_ts = [all_ts]
@@ -101,9 +102,9 @@ class Solution(object):
         self._y_event = y_event
         self._termination = termination
 
-        self.has_symbolic_inputs = any(
-            isinstance(v, casadi.MX) for v in self.all_inputs[0].values()
-        )
+        # Check no ys are too large
+        if check_solution:
+            self.check_ys_are_not_too_large()
 
         # Copy the timescale_eval and lengthscale_evals if they exist
         if hasattr(all_models[0], "timescale_eval"):
@@ -130,7 +131,7 @@ class Solution(object):
         self.solve_time = None
         self.integration_time = None
 
-        # initiaize empty variables and data
+        # initialize empty variables and data
         self._variables = pybamm.FuzzyDict()
         self.data = pybamm.FuzzyDict()
 
@@ -147,7 +148,7 @@ class Solution(object):
         pybamm.citations.register("Andersson2019")
 
     def extract_explicit_sensitivities(self):
-        # if we got here, we havn't set y yet
+        # if we got here, we haven't set y yet
         self.set_y()
 
         # extract sensitivities from full y solution
@@ -312,6 +313,24 @@ class Solution(object):
                 "computed explicitly."
             )
 
+    def check_ys_are_not_too_large(self):
+        # Only check last one so that it doesn't take too long
+        # We only care about the cases where y is growing too large without any
+        # restraint, so if y gets large in the middle then comes back down that is ok
+        y, model = self.all_ys[-1], self.all_models[-1]
+        y = y[:, -1]
+        if np.any(y > pybamm.settings.max_y_value):
+            for var in [*model.rhs.keys(), *model.algebraic.keys()]:
+                y_var = y[model.variables[var.name].y_slices[0]]
+                if np.any(y_var > pybamm.settings.max_y_value):
+                    pybamm.logger.error(
+                        f"Solution for '{var}' exceeds the maximum allowed value "
+                        f"of `{pybamm.settings.max_y_value}. This could be due to "
+                        "incorrect nondimensionalisation, model formulation, or "
+                        "parameter values. The maximum allowed value is set by "
+                        "'pybammm.settings.max_y_value'."
+                    )
+
     @property
     def all_ts(self):
         return self._all_ts
@@ -452,55 +471,48 @@ class Solution(object):
             variables = [variables]
         # Process
         for key in variables:
+            cumtrapz_ic = None
             pybamm.logger.debug("Post-processing {}".format(key))
-            # If there are symbolic inputs then we need to make a
-            # ProcessedSymbolicVariable
-            if self.has_symbolic_inputs is True:
-                var = pybamm.ProcessedSymbolicVariable(
-                    self.all_models[0].variables[key], self
-                )
+            vars_pybamm = [model.variables_and_events[key] for model in self.all_models]
 
-            # Otherwise a standard ProcessedVariable is ok
-            else:
-                vars_pybamm = [model.variables[key] for model in self.all_models]
-
-                # Iterate through all models, some may be in the list several times and
-                # therefore only get set up once
-                vars_casadi = []
-                for model, ys, inputs, var_pybamm in zip(
-                    self.all_models, self.all_ys, self.all_inputs, vars_pybamm
-                ):
-                    if key in model._variables_casadi:
-                        var_casadi = model._variables_casadi[key]
-                    else:
-                        t_MX = casadi.MX.sym("t")
-                        y_MX = casadi.MX.sym("y", ys.shape[0])
-                        symbolic_inputs_dict = {
-                            key: casadi.MX.sym("input", value.shape[0])
-                            for key, value in inputs.items()
-                        }
-                        symbolic_inputs = casadi.vertcat(
-                            *[p for p in symbolic_inputs_dict.values()]
-                        )
-
-                        # Convert variable to casadi
-                        # Make all inputs symbolic first for converting to casadi
-                        var_sym = var_pybamm.to_casadi(
-                            t_MX, y_MX, inputs=symbolic_inputs_dict
-                        )
-
-                        var_casadi = casadi.Function(
-                            "variable", [t_MX, y_MX, symbolic_inputs], [var_sym]
-                        )
-                        model._variables_casadi[key] = var_casadi
-                    vars_casadi.append(var_casadi)
-
-                var = pybamm.ProcessedVariable(vars_pybamm, vars_casadi, self)
+            # Iterate through all models, some may be in the list several times and
+            # therefore only get set up once
+            vars_casadi = []
+            for (i, (model, ys, inputs, var_pybamm)) in enumerate(
+                zip(self.all_models, self.all_ys, self.all_inputs, vars_pybamm)
+            ):
+                if isinstance(var_pybamm, pybamm.ExplicitTimeIntegral):
+                    cumtrapz_ic = var_pybamm.initial_condition
+                    cumtrapz_ic = cumtrapz_ic.evaluate()
+                    var_pybamm = var_pybamm.child
+                    var_casadi = self.process_casadi_var(var_pybamm, inputs, ys)
+                    model._variables_casadi[key] = var_casadi
+                    vars_pybamm[i] = var_pybamm
+                elif key in model._variables_casadi:
+                    var_casadi = model._variables_casadi[key]
+                else:
+                    var_casadi = self.process_casadi_var(var_pybamm, inputs, ys)
+                    model._variables_casadi[key] = var_casadi
+                vars_casadi.append(var_casadi)
+            var = pybamm.ProcessedVariable(
+                vars_pybamm, vars_casadi, self, cumtrapz_ic=cumtrapz_ic
+            )
 
             # Save variable and data
             self._variables[key] = var
             self.data[key] = var.data
 
+    def process_casadi_var(self, var_pybamm, inputs, ys):
+        t_MX = casadi.MX.sym("t")
+        y_MX = casadi.MX.sym("y", ys.shape[0])
+        inputs_MX_dict = {
+            key: casadi.MX.sym("input", value.shape[0]) for key, value in inputs.items()
+        }
+        inputs_MX = casadi.vertcat(*[p for p in inputs_MX_dict.values()])
+        var_sym = var_pybamm.to_casadi(t_MX, y_MX, inputs=inputs_MX_dict)
+        var_casadi = casadi.Function("variable", [t_MX, y_MX, inputs_MX], [var_sym])
+        return var_casadi
+
     def __getitem__(self, key):
         """Read a variable from the solution. Variables are created 'just in time', i.e.
         only when they are called.
@@ -671,6 +683,8 @@ class Solution(object):
 
     def __add__(self, other):
         """Adds two solutions together, e.g. when stepping"""
+        if other is None or isinstance(other, EmptySolution):
+            return self.copy()
         if not isinstance(other, Solution):
             raise pybamm.SolverError(
                 "Only a Solution or None can be added to a Solution"
@@ -722,16 +736,7 @@ class Solution(object):
         return new_sol
 
     def __radd__(self, other):
-        """
-        Right-side adding with special handling for the case None + Solution (returns
-        Solution)
-        """
-        if other is None:
-            return self.copy()
-        else:
-            raise pybamm.SolverError(
-                "Only a Solution or None can be added to a Solution"
-            )
+        return self.__add__(other)
 
     def copy(self):
         new_sol = self.__class__(
@@ -754,7 +759,29 @@ class Solution(object):
         return new_sol
 
 
-def make_cycle_solution(step_solutions, esoh_sim=None, save_this_cycle=True):
+class EmptySolution:
+    def __init__(self, termination=None, t=None):
+        self.termination = termination
+        if t is None:
+            t = np.array([0])
+        elif isinstance(t, numbers.Number):
+            t = np.array([t])
+
+        self.t = t
+
+    def __add__(self, other):
+        if isinstance(other, (EmptySolution, Solution)):
+            return other.copy()
+
+    def __radd__(self, other):
+        if other is None:
+            return self.copy()
+
+    def copy(self):
+        return EmptySolution(self.termination)
+
+
+def make_cycle_solution(step_solutions, esoh_solver=None, save_this_cycle=True):
     """
     Function to create a Solution for an entire cycle, and associated summary variables
 
@@ -762,10 +789,9 @@ def make_cycle_solution(step_solutions, esoh_sim=None, save_this_cycle=True):
     ----------
     step_solutions : list of :class:`Solution`
         Step solutions that form the entire cycle
-    esoh_sim : :class:`pybamm.Simulation`, optional
-        A simulation, whose model should be a :class:`pybamm.lithium_ion.ElectrodeSOH`
-        model, which is used to calculate some of the summary variables. If `None`
-        (default) then only summary variables that do not require the eSOH calculation
+    esoh_solver : :class:`pybamm.lithium_ion.ElectrodeSOHSolver`
+        Solver to calculate electrode SOH (eSOH) variables. If `None` (default)
+        then only summary variables that do not require the eSOH calculation
         are calculated. See [1] for more details on eSOH variables.
     save_this_cycle : bool, optional
         Whether to save the entire cycle variables or just the summary variables.
@@ -807,7 +833,7 @@ def make_cycle_solution(step_solutions, esoh_sim=None, save_this_cycle=True):
 
     cycle_solution.steps = step_solutions
 
-    cycle_summary_variables = get_cycle_summary_variables(cycle_solution, esoh_sim)
+    cycle_summary_variables = _get_cycle_summary_variables(cycle_solution, esoh_solver)
 
     cycle_first_state = cycle_solution.first_state
 
@@ -819,15 +845,14 @@ def make_cycle_solution(step_solutions, esoh_sim=None, save_this_cycle=True):
     return cycle_solution, cycle_summary_variables, cycle_first_state
 
 
-def get_cycle_summary_variables(cycle_solution, esoh_sim):
+def _get_cycle_summary_variables(cycle_solution, esoh_solver):
     model = cycle_solution.all_models[0]
     cycle_summary_variables = pybamm.FuzzyDict({})
 
     # Measured capacity variables
     if "Discharge capacity [A.h]" in model.variables:
         Q = cycle_solution["Discharge capacity [A.h]"].data
-        min_Q = np.min(Q)
-        max_Q = np.max(Q)
+        min_Q, max_Q = np.min(Q), np.max(Q)
 
         cycle_summary_variables.update(
             {
@@ -837,6 +862,15 @@ def get_cycle_summary_variables(cycle_solution, esoh_sim):
             }
         )
 
+    # Voltage variables
+    if "Battery voltage [V]" in model.variables:
+        V = cycle_solution["Battery voltage [V]"].data
+        min_V, max_V = np.min(V), np.max(V)
+
+        cycle_summary_variables.update(
+            {"Minimum voltage [V]": min_V, "Maximum voltage [V]": max_V}
+        )
+
     # Degradation variables
     degradation_variables = model.summary_variables
     first_state = cycle_solution.first_state
@@ -852,50 +886,16 @@ def get_cycle_summary_variables(cycle_solution, esoh_sim):
 
     # eSOH variables (full-cell lithium-ion model only, for now)
     if (
-        esoh_sim is not None
+        esoh_solver is not None
         and isinstance(model, pybamm.lithium_ion.BaseModel)
-        and model.half_cell is False
+        and model.options.electrode_types["negative"] == "porous"
     ):
-        V_min = esoh_sim.parameter_values["Lower voltage cut-off [V]"]
-        V_max = esoh_sim.parameter_values["Upper voltage cut-off [V]"]
+        V_min = esoh_solver.parameter_values["Lower voltage cut-off [V]"]
+        V_max = esoh_solver.parameter_values["Upper voltage cut-off [V]"]
         C_n = last_state["Negative electrode capacity [A.h]"].data[0]
         C_p = last_state["Positive electrode capacity [A.h]"].data[0]
         n_Li = last_state["Total lithium in particles [mol]"].data[0]
-        if esoh_sim.solution is not None:
-            # initialize with previous solution if it is available
-            esoh_sim.built_model.set_initial_conditions_from(esoh_sim.solution)
-            solver = None
-        else:
-            x_100_init = np.max(cycle_solution["Negative electrode SOC"].data)
-            # make sure x_0 > 0
-            C_init = np.minimum(0.95 * (C_n * x_100_init), max_Q - min_Q)
 
-            # Solve the esoh model and add outputs to the summary variables
-            # use CasadiAlgebraicSolver if there are interpolants
-            if isinstance(
-                esoh_sim.parameter_values["Negative electrode OCP [V]"], tuple
-            ) or isinstance(
-                esoh_sim.parameter_values["Positive electrode OCP [V]"], tuple
-            ):
-                solver = pybamm.CasadiAlgebraicSolver()
-                # Choose x_100_init so as not to violate the interpolation limits
-                if isinstance(
-                    esoh_sim.parameter_values["Positive electrode OCP [V]"], tuple
-                ):
-                    y_100_min = np.min(
-                        esoh_sim.parameter_values["Positive electrode OCP [V]"][1][0][0]
-                    )
-                    x_100_max = (
-                        n_Li * pybamm.constants.F.value / 3600 - y_100_min * C_p
-                    ) / C_n
-                    x_100_init = np.minimum(x_100_init, 0.99 * x_100_max)
-            else:
-                solver = None
-            # Update initial conditions using the cycle solution
-            esoh_sim.build()
-            esoh_sim.built_model.set_initial_conditions_from(
-                {"x_100": x_100_init, "C": C_init}
-            )
         inputs = {
             "V_min": V_min,
             "V_max": V_max,
@@ -905,15 +905,14 @@ def get_cycle_summary_variables(cycle_solution, esoh_sim):
         }
 
         try:
-            esoh_sol = esoh_sim.solve([0], inputs=inputs, solver=solver)
+            esoh_sol = esoh_solver.solve(inputs)
         except pybamm.SolverError:  # pragma: no cover
             raise pybamm.SolverError(
                 "Could not solve for summary variables, run "
                 "`sim.solve(calc_esoh=False)` to skip this step"
             )
-        for var in esoh_sim.built_model.variables:
+
+        for var in esoh_sol.all_models[0].variables:
             cycle_summary_variables[var] = esoh_sol[var].data[0]
 
-        cycle_summary_variables["Capacity [A.h]"] = cycle_summary_variables["C"]
-
     return cycle_summary_variables
