@@ -13,8 +13,9 @@ import pickle
 import subprocess
 import sys
 import timeit
-from collections import defaultdict
 from platform import system
+import difflib
+from julia.api import Julia, JuliaInfo, JuliaError
 
 import numpy as np
 import pkg_resources
@@ -31,85 +32,32 @@ def root_dir():
     return str(pathlib.Path(pybamm.__path__[0]).parent)
 
 
+def get_git_commit_info():
+    """
+    Get the git commit info for the current PyBaMM version, e.g. v22.8-39-gb25ce8c41
+    (version 22.8, commit b25ce8c41)
+    """
+    try:
+        # Get the latest git commit hash
+        return str(
+            subprocess.check_output(["git", "describe", "--tags"], cwd=root_dir())
+            .strip()
+            .decode()
+        )
+    except subprocess.CalledProcessError:  # pragma: no cover
+        # Not a git repository so just return the version number
+        return f"v{pybamm.__version__}"
+
+
 class FuzzyDict(dict):
-    def levenshtein_ratio(self, s, t):
-        """
-        Calculates levenshtein distance between two strings s and t.
-        Uses the formula from
-        https://www.datacamp.com/community/tutorials/fuzzy-string-python
-        """
-        # Initialize matrix of zeros
-        rows = len(s) + 1
-        cols = len(t) + 1
-        distance = np.zeros((rows, cols), dtype=int)
-
-        # Populate matrix of zeros with the indices of each character of both strings
-        for i in range(1, rows):
-            for k in range(1, cols):
-                distance[i][0] = i
-                distance[0][k] = k
-
-        # Iterate over the matrix to compute the cost of deletions, insertions and/or
-        # substitutions
-        for col in range(1, cols):
-            for row in range(1, rows):
-                if s[row - 1] == t[col - 1]:
-                    # If the characters are the same in the two strings in a given
-                    # position [i,j] then the cost is 0
-                    cost = 0
-                else:
-                    # In order to align the results with those of the Python Levenshtein
-                    # package, the cost of a substitution is 2.
-                    cost = 2
-                distance[row][col] = min(
-                    distance[row - 1][col] + 1,  # Cost of deletions
-                    distance[row][col - 1] + 1,  # Cost of insertions
-                    distance[row - 1][col - 1] + cost,  # Cost of substitutions
-                )
-        # Computation of the Levenshtein Distance Ratio
-        ratio = ((len(s) + len(t)) - distance[row][col]) / (len(s) + len(t))
-        return ratio
-
     def get_best_matches(self, key):
         """Get best matches from keys"""
-        key = key.lower()
-        best_three = []
-        lowest_score = 0
-        for k in self.keys():
-            score = self.levenshtein_ratio(k.lower(), key)
-            # Start filling out the list
-            if len(best_three) < 3:
-                best_three.append((k, score))
-                # Sort once the list has three elements, using scores
-                if len(best_three) == 3:
-                    best_three.sort(key=lambda x: x[1], reverse=True)
-                    lowest_score = best_three[-1][1]
-            # Once list is full, start checking new entries
-            else:
-                if score > lowest_score:
-                    # Replace last element with new entry
-                    best_three[-1] = (k, score)
-                    # Sort and update lowest score
-                    best_three.sort(key=lambda x: x[1], reverse=True)
-                    lowest_score = best_three[-1][1]
-
-        return [x[0] for x in best_three]
+        return difflib.get_close_matches(key, list(self.keys()), n=3, cutoff=0.5)
 
     def __getitem__(self, key):
         try:
             return super().__getitem__(key)
         except KeyError:
-            if "negative electrode sei" in key.lower():
-                raise KeyError(
-                    f"'{key}' not found. All SEI parameters have been "
-                    "renamed from '...negative electrode SEI...' to '...SEI...'"
-                )
-            if "negative electrode lithium plating" in key.lower():
-                raise KeyError(
-                    f"'{key}' not found. All lithium plating parameters have been "
-                    "renamed from '...negative electrode lithium plating...' "
-                    "to '...lithium plating...'"
-                )
             best_matches = self.get_best_matches(key)
             raise KeyError(f"'{key}' not found. Best matches are {best_matches}")
 
@@ -119,7 +67,8 @@ class FuzzyDict(dict):
         both the keys and values will be printed. Otherwise just the values will
         be printed. If no results are found, the best matches are printed.
         """
-        key = key.lower()
+        key_in = key
+        key = key_in.lower()
 
         # Sort the keys so results are stored in alphabetical order
         keys = list(self.keys())
@@ -135,7 +84,8 @@ class FuzzyDict(dict):
             # If no results, return best matches
             best_matches = self.get_best_matches(key)
             print(
-                f"No results for search using '{key}'. Best matches are {best_matches}"
+                f"No results for search using '{key_in}'. "
+                f"Best matches are {best_matches}"
             )
         elif print_values:
             # Else print results, including dict items
@@ -144,6 +94,9 @@ class FuzzyDict(dict):
             # Just print keys
             print("\n".join("{}".format(k) for k in results.keys()))
 
+    def copy(self):
+        return FuzzyDict(super().copy())
+
 
 class Timer(object):
     """
@@ -203,6 +156,9 @@ class TimerTime:
         output.append("1 second" if time == 1 else str(time) + " seconds")
         return ", ".join(output)
 
+    def __repr__(self):
+        return f"pybamm.TimerTime({self.value})"
+
     def __add__(self, other):
         if isinstance(other, numbers.Number):
             return TimerTime(self.value + other)
@@ -245,7 +201,7 @@ class TimerTime:
         return self.value == other.value
 
 
-def load_function(filename):
+def load_function(filename, funcname=None):
     """
     Load a python function from an absolute or relative path using `importlib`.
     Example - pybamm.load_function("pybamm/input/example.py")
@@ -254,6 +210,9 @@ def load_function(filename):
     ---------
     filename : str
         The path of the file containing the function.
+    funcname : str, optional
+        The name of the function in the file. If None, assumed to be the same as the
+        filename (ignoring the path)
 
     Returns
     -------
@@ -264,8 +223,9 @@ def load_function(filename):
     if filename.endswith(".py"):
         filename = filename.replace(".py", "")
 
-    # Assign path to _ and filename to tail
-    _, tail = os.path.split(filename)
+    if funcname is None:
+        # Read funcname by splitting the file (assumes funcname is the same as filename)
+        _, funcname = os.path.split(filename)
 
     # Store the current working directory
     orig_dir = os.getcwd()
@@ -274,7 +234,7 @@ def load_function(filename):
     if "pybamm/input/parameters" in filename or "pybamm\\input\\parameters" in filename:
         root_path = filename[filename.rfind("pybamm") :]
     # If the function is in the current working directory
-    elif os.getcwd() in filename:
+    elif os.getcwd() in filename:  # pragma: no cover
         root_path = filename.replace(os.getcwd(), "")
     # If the function is not in the current working directory and the path provided is
     # absolute
@@ -283,23 +243,23 @@ def load_function(filename):
         dir_path = os.path.split(filename)[0]
         os.chdir(dir_path)
         root_path = filename.replace(os.getcwd(), "")
-    else:
+    else:  # pragma: no cover
         root_path = filename
 
     # getcwd() returns "C:\\" when in the root drive and "C:\\a\\b\\c" otherwise
-    if root_path[0] == "\\" or root_path[0] == "/":
+    if root_path[0] == "\\" or root_path[0] == "/":  # pragma: no cover
         root_path = root_path[1:]
 
     path = root_path.replace("/", ".")
     path = path.replace("\\", ".")
     pybamm.logger.debug(
-        f"Importing function '{tail}' from file '{filename}' via path '{path}'"
+        f"Importing function '{funcname}' from file '{filename}' via path '{path}'"
     )
     module_object = importlib.import_module(path)
 
     # Revert back current working directory if it was changed
     os.chdir(orig_dir)
-    return getattr(module_object, tail)
+    return getattr(module_object, funcname)
 
 
 def rmse(x, y):
@@ -312,28 +272,6 @@ def rmse(x, y):
     return np.sqrt(np.nanmean((x - y) ** 2))
 
 
-def get_infinite_nested_dict():
-    """
-    Return a dictionary that allows infinite nesting without having to define level by
-    level.
-
-    See:
-    https://stackoverflow.com/questions/651794/whats-the-best-way-to-initialize-a-dict-of-dicts-in-python/652226#652226
-
-    Example
-    -------
-    >>> import pybamm
-    >>> d = pybamm.get_infinite_nested_dict()
-    >>> d["a"] = 1
-    >>> d["a"]
-    1
-    >>> d["b"]["c"]["d"] = 2
-    >>> d["b"]["c"] == {"d": 2}
-    True
-    """
-    return defaultdict(get_infinite_nested_dict)
-
-
 def load(filename):
     """Load a saved object"""
     with open(filename, "rb") as f:
@@ -350,6 +288,31 @@ def get_parameters_filepath(path):
         return os.path.join(pybamm.__path__[0], path)
 
 
+def have_julia():
+    """
+    Checks whether the Julia programming language has been installed
+    """
+
+    # Try fetching info about julia
+    try:
+        info = JuliaInfo.load()
+    except (FileNotFoundError, subprocess.CalledProcessError):
+        return False
+
+    # Compatibility: Checks
+    if not info.is_pycall_built():  # pragma: no cover
+        return False
+    if not info.is_compatible_python():  # pragma: no cover
+        return False
+
+    # Confirm Julia() is callable
+    try:
+        Julia()
+        return True
+    except JuliaError:  # pragma: no cover
+        return False
+
+
 def have_jax():
     """Check if jax and jaxlib are installed with the correct versions"""
     return (
@@ -367,6 +330,22 @@ def is_jax_compatible():
     )
 
 
+def is_constant_and_can_evaluate(symbol):
+    """
+    Returns True if symbol is constant and evaluation does not raise any errors.
+    Returns False otherwise.
+    An example of a constant symbol that cannot be "evaluated" is PrimaryBroadcast(0).
+    """
+    if symbol.is_constant():
+        try:
+            symbol.evaluate()
+            return True
+        except NotImplementedError:
+            return False
+    else:
+        return False
+
+
 def install_jax(arguments=None):  # pragma: no cover
     """
     Install compatible versions of jax, jaxlib.
